#include "mtime_timedelta_test.h"

#include <stdio.h>
#include <stdlib.h>
#include "mtime_calendar.h"
#include "mtime_datetime.h"
#include "mtime_julianDay.h"
#include "mtime_date.h"

typedef struct _timedelta* TimeDelta;
typedef struct _datetime* DateTime;
typedef struct _juliandelta* JulianDelta;
typedef struct _divisionquotienttimedelta* DivisionQuotientTimeDelta;
typedef struct _date* Date;


START_TEST(test_create_timedelta_from_valid_strings)
{
	assertTimeDelta("P0Y", true, '+', 0, 0, 0, 0, 0, 0, 0, "PT00.000S");
	assertTimeDelta("P0M", true, '+', 0, 0, 0, 0, 0, 0, 0, "PT00.000S");
	assertTimeDelta("P0D", true, '+', 0, 0, 0, 0, 0, 0, 0, "PT00.000S");
	assertTimeDelta("PT0H", true, '+', 0, 0, 0, 0, 0, 0, 0, "PT00.000S");
	assertTimeDelta("PT0M", true, '+', 0, 0, 0, 0, 0, 0, 0, "PT00.000S");
	assertTimeDelta("PT0S", true, '+', 0, 0, 0, 0, 0, 0, 0, "PT00.000S");

	assertTimeDelta("P42Y", true, '+', 42, 0, 0, 0, 0, 0, 0, "P42Y");
	assertTimeDelta("P6M", true, '+', 0, 6, 0, 0, 0, 0, 0, "P06M");
	assertTimeDelta("P23D", true, '+', 0, 0, 23, 0, 0, 0, 0, "P23D");
	assertTimeDelta("P109D", false, '+', 0, 0, 109, 0, 0, 0, 0, "P109D");
	assertTimeDelta("+P109D", false, '+', 0, 0, 109, 0, 0, 0, 0, "P109D");
	assertTimeDelta("PT13H", true, '+', 0, 0, 0, 13, 0, 0, 0, "PT13H");
	assertTimeDelta("PT36H", false, '+', 0, 0, 0, 36, 0, 0, 0, "PT36H");
	assertTimeDelta("PT0036H", false, '+', 0, 0, 0, 36, 0, 0, 0, "PT36H");
	assertTimeDelta("PT15M", true, '+', 0, 0, 0, 0, 15, 0, 0, "PT15M");
	assertTimeDelta("-PT15M", true, '-', 0, 0, 0, 0, 15, 0, 0, "-PT15M");
	assertTimeDelta("PT75M", false, '+', 0, 0, 0, 0, 75, 0, 0, "PT75M");
	assertTimeDelta("PT8S", true, '+', 0, 0, 0, 0, 0, 8, 0, "PT08.000S");
	assertTimeDelta("PT08S", true, '+', 0, 0, 0, 0, 0, 8, 0, "PT08.000S");
	assertTimeDelta("+PT8S", true, '+', 0, 0, 0, 0, 0, 8, 0, "PT08.000S");
	assertTimeDelta("PT57S", true, '+', 0, 0, 0, 0, 0, 57, 0, "PT57.000S");
	assertTimeDelta("PT1024S", false, '+', 0, 0, 0, 0, 0, 1024, 0, "PT1024.000S");
	assertTimeDelta("PT2147483647S", false, '+', 0, 0, 0, 0, 0, 2147483647, 0, "PT2147483647.000S");
	assertTimeDelta("PT2147483647.999S", false, '+', 0, 0, 0, 0, 0, 2147483647, 999, "PT2147483647.999S");
	assertTimeDelta("-PT1024S", false, '-', 0, 0, 0, 0, 0, 1024, 0, "-PT1024.000S");
	assertTimeDelta("PT0.033S", true, '+', 0, 0, 0, 0, 0, 0, 33, "PT00.033S");

	assertTimeDelta("P504Y4M12D", true, '+', 504, 4, 12, 0, 0, 0, 0, "P504Y04M12D");
	assertTimeDelta("+P504Y4M12D", true, '+', 504, 4, 12, 0, 0, 0, 0, "P504Y04M12D");
	assertTimeDelta("-P504Y4M12D", true, '-', 504, 4, 12, 0, 0, 0, 0, "-P504Y04M12D");

	assertTimeDelta("PT18H1M13.123S", true, '+', 0, 0, 0, 18, 1, 13, 123, "PT18H01M13.123S");
	assertTimeDelta("+PT18H1M13.123S", true, '+', 0, 0, 0, 18, 1, 13, 123, "PT18H01M13.123S");
	assertTimeDelta("-PT18H1M13.123S", true, '-', 0, 0, 0, 18, 1, 13, 123, "-PT18H01M13.123S");

	assertTimeDelta("PT1M13.123S", true, '+', 0, 0, 0, 0, 1, 13, 123, "PT01M13.123S");
	assertTimeDelta("PT18H1M13.123S", true, '+', 0, 0, 0, 18, 1, 13, 123, "PT18H01M13.123S");
	assertTimeDelta("P12DT18H1M13.123S", true, '+', 0, 0, 12, 18, 1, 13, 123, "P12DT18H01M13.123S");
	assertTimeDelta("P4M12DT18H1M13.123S", true, '+', 0, 4, 12, 18, 1, 13, 123, "P04M12DT18H01M13.123S");
	assertTimeDelta("P504Y4M12DT18H1M13.123S", true, '+', 504, 4, 12, 18, 1, 13, 123, "P504Y04M12DT18H01M13.123S");
	assertTimeDelta("+P504Y4M12DT18H1M13.123S", true, '+', 504, 4, 12, 18, 1, 13, 123, "P504Y04M12DT18H01M13.123S");
	assertTimeDelta("-P504Y4M12DT18H1M13.123S", true, '-', 504, 4, 12, 18, 1, 13, 123, "-P504Y04M12DT18H01M13.123S");
	assertTimeDelta("-P504Y4M12DT18H1M13.000S", true, '-', 504, 4, 12, 18, 1, 13, 000, "-P504Y04M12DT18H01M13.000S");
	assertTimeDelta("-P504Y4M12DT18H1M", true, '-', 504, 4, 12, 18, 1, 0, 000, "-P504Y04M12DT18H01M");
	assertTimeDelta("-P504Y4M12DT18H", true, '-', 504, 4, 12, 18, 0, 0, 000, "-P504Y04M12DT18H");
	assertTimeDelta("-P504Y4M12D", true, '-', 504, 4, 12, 0, 0, 0, 000, "-P504Y04M12D");
	assertTimeDelta("-P504Y4M", true, '-', 504, 4, 0, 0, 0, 0, 000, "-P504Y04M");
	assertTimeDelta("P4M12DT18H1M", true, '+', 0, 4, 12, 18, 1, 0, 0, "P04M12DT18H01M");
	assertTimeDelta("P12DT18H", true, '+', 0, 0, 12, 18, 0, 0, 0, "P12DT18H");

	assertTimeDelta("P0Y0M0DT0H0M0.000S", true, '+', 0, 0, 0, 0, 0, 0, 0, "PT00.000S");
	assertTimeDelta("P0YT0H", true, '+', 0, 0, 0, 0, 0, 0, 0, "PT00.000S");
	assertTimeDelta("P1Y1M1DT1H1M1.001S", true, '+', 1, 1, 1, 1, 1, 1, 1, "P1Y01M01DT01H01M01.001S");
	assertTimeDelta("P2147483647Y", true, '+', 2147483647, 0, 0, 0, 0, 0, 0, "P2147483647Y");
	assertTimeDelta("-P2147483648Y", true, '-', 2147483648, 0, 0, 0, 0, 0, 0, "-P2147483648Y");
	assertTimeDelta("P2147483647D", false, '+', 0, 0, 2147483647, 0, 0, 0, 0, "P2147483647D");
	assertTimeDelta("-P2147483647D", false, '-', 0, 0, 2147483647, 0, 0, 0, 0, "-P2147483647D");
	assertTimeDelta("PT2147483647H", false, '+', 0, 0, 0, 2147483647, 0, 0, 0, "PT2147483647H");
	assertTimeDelta("-PT2147483647H", false, '-', 0, 0, 0, 2147483647, 0, 0, 0, "-PT2147483647H");
	assertTimeDelta("PT2147483647M", false, '+', 0, 0, 0, 0, 2147483647, 0, 0, "PT2147483647M");
	assertTimeDelta("-PT2147483647M", false, '-', 0, 0, 0, 0, 2147483647, 0, 0, "-PT2147483647M");
	assertTimeDelta("PT2147483647S", false, '+', 0, 0, 0, 0, 0, 2147483647, 0, "PT2147483647.000S");
	assertTimeDelta("-PT2147483647S", false, '-', 0, 0, 0, 0, 0, 2147483647, 0, "-PT2147483647.000S");
	assertTimeDelta("PT0.999S", true, '+', 0, 0, 0, 0, 0, 0, 999, "PT00.999S");
	assertTimeDelta("-PT0.999S", true, '-', 0, 0, 0, 0, 0, 0, 999, "-PT00.999S");
}
END_TEST

START_TEST(test_invalid_strings)
{
	assertInvalidTimeDelta(NULL);
	assertInvalidTimeDelta("");
	assertInvalidTimeDelta("abc");
	assertInvalidTimeDelta("P3Z");
	assertInvalidTimeDelta("P42M");
	// And many more...
}
END_TEST

START_TEST(test_constructAndCopyTimeDelta)
{
	int64_t year = 504;
	int month = 4;
	int day = 12;
	int hour = 18;
	int minute = 1;
	int second = 13;
	int ms = 123;

	TimeDelta timedelta_src = newRawTimeDelta('+', year, month, day, hour, minute, second, ms);
	TimeDelta timedelta_dest = constructAndCopyTimeDelta(timedelta_src);
	ck_assert(timedelta_dest != NULL);
	ck_assert(timedelta_dest->flag_std_form);
	ck_assert_int_eq('+', timedelta_dest->sign);
	ck_assert_int_eq(year, timedelta_dest->year);
	ck_assert_int_eq(month, timedelta_dest->month);
	ck_assert_int_eq(day, timedelta_dest->day);
	ck_assert_int_eq(hour, timedelta_dest->hour);
	ck_assert_int_eq(minute, timedelta_dest->minute);
	ck_assert_int_eq(second, timedelta_dest->second);
	ck_assert_int_eq(ms, timedelta_dest->ms);

	TimeDelta timedelta_src_neg = newRawTimeDelta('-', year, month, day, hour, minute, second, ms);
	TimeDelta timedelta_dest_neg = constructAndCopyTimeDelta(timedelta_src_neg);
	ck_assert(timedelta_dest_neg != NULL);
	ck_assert(timedelta_dest_neg->flag_std_form);
	ck_assert_int_eq('-', timedelta_dest_neg->sign);
	ck_assert_int_eq(year, timedelta_dest_neg->year);
	ck_assert_int_eq(month, timedelta_dest_neg->month);
	ck_assert_int_eq(day, timedelta_dest_neg->day);
	ck_assert_int_eq(hour, timedelta_dest_neg->hour);
	ck_assert_int_eq(minute, timedelta_dest_neg->minute);
	ck_assert_int_eq(second, timedelta_dest_neg->second);
	ck_assert_int_eq(ms, timedelta_dest_neg->ms);

	TimeDelta timedelta_src_nonstd = newRawTimeDelta('-', 0, 0, 2000, 0, 0, 0, 0);
	TimeDelta timedelta_dest_nonstd = constructAndCopyTimeDelta(timedelta_src_nonstd);
	ck_assert(timedelta_dest_nonstd != NULL);
	ck_assert(!timedelta_dest_nonstd->flag_std_form);
	ck_assert_int_eq('-', timedelta_dest_nonstd->sign);
	ck_assert_int_eq(0, timedelta_dest_nonstd->year);
	ck_assert_int_eq(0, timedelta_dest_nonstd->month);
	ck_assert_int_eq(2000, timedelta_dest_nonstd->day);
	ck_assert_int_eq(0, timedelta_dest_nonstd->hour);
	ck_assert_int_eq(0, timedelta_dest_nonstd->minute);
	ck_assert_int_eq(0, timedelta_dest_nonstd->second);
	ck_assert_int_eq(0, timedelta_dest_nonstd->ms);

	deallocateTimeDelta(timedelta_src);
	deallocateTimeDelta(timedelta_dest);
	deallocateTimeDelta(timedelta_src_neg);
	deallocateTimeDelta(timedelta_dest_neg);
	deallocateTimeDelta(timedelta_src_nonstd);
	deallocateTimeDelta(timedelta_dest_nonstd);
}
END_TEST

START_TEST(test_replaceTimeDelta)
{
	int64_t year = 504;
	int month = 4;
	int day = 12;
	int hour = 18;
	int minute = 1;
	int second = 13;
	int ms = 123;

	TimeDelta timedelta_src = newRawTimeDelta('+', year, month, day, hour, minute, second, ms);
	TimeDelta timedelta_dest = newTimeDelta("PT0S");
	replaceTimeDelta(timedelta_src, timedelta_dest);
	ck_assert(timedelta_dest != NULL);
	ck_assert_int_eq('+', timedelta_dest->sign);
	ck_assert_int_eq(year, timedelta_dest->year);
	ck_assert_int_eq(month, timedelta_dest->month);
	ck_assert_int_eq(day, timedelta_dest->day);
	ck_assert_int_eq(hour, timedelta_dest->hour);
	ck_assert_int_eq(minute, timedelta_dest->minute);
	ck_assert_int_eq(second, timedelta_dest->second);
	ck_assert_int_eq(ms, timedelta_dest->ms);

	TimeDelta timedelta_src_neg = newRawTimeDelta('-', year, month, day, hour, minute, second, ms);
	TimeDelta timedelta_dest_neg = newTimeDelta("P0Y");
	replaceTimeDelta(timedelta_src_neg, timedelta_dest_neg);
	ck_assert(timedelta_dest_neg != NULL);
	ck_assert_int_eq('-', timedelta_dest_neg->sign);
	ck_assert_int_eq(year, timedelta_dest_neg->year);
	ck_assert_int_eq(month, timedelta_dest_neg->month);
	ck_assert_int_eq(day, timedelta_dest_neg->day);
	ck_assert_int_eq(hour, timedelta_dest_neg->hour);
	ck_assert_int_eq(minute, timedelta_dest_neg->minute);
	ck_assert_int_eq(second, timedelta_dest_neg->second);
	ck_assert_int_eq(ms, timedelta_dest_neg->ms);

	TimeDelta timedelta_src_nonstd = newRawTimeDelta('-', 0, 0, 2000, 0, 0, 0, 0);
	TimeDelta timedelta_dest_nonstd = newTimeDelta("P0D");
	replaceTimeDelta(timedelta_src_nonstd, timedelta_dest_nonstd);
	ck_assert(timedelta_dest_nonstd != NULL);
	ck_assert(!timedelta_dest_nonstd->flag_std_form);
	ck_assert_int_eq('-', timedelta_dest_nonstd->sign);
	ck_assert_int_eq(0, timedelta_dest_nonstd->year);
	ck_assert_int_eq(0, timedelta_dest_nonstd->month);
	ck_assert_int_eq(2000, timedelta_dest_nonstd->day);
	ck_assert_int_eq(0, timedelta_dest_nonstd->hour);
	ck_assert_int_eq(0, timedelta_dest_nonstd->minute);
	ck_assert_int_eq(0, timedelta_dest_nonstd->second);
	ck_assert_int_eq(0, timedelta_dest_nonstd->ms);

	deallocateTimeDelta(timedelta_src);
	deallocateTimeDelta(timedelta_dest);
	deallocateTimeDelta(timedelta_src_neg);
	deallocateTimeDelta(timedelta_dest_neg);
	deallocateTimeDelta(timedelta_src_nonstd);
	deallocateTimeDelta(timedelta_dest_nonstd);
}
END_TEST

START_TEST(test_timeDeltaToJulianDelta_ProlepticGregorian_1)
{
	assertTimeDeltaToJulianDelta("P0Y", "0-01-01T00:00:00.000", '+', 0, 0);

	assertTimeDeltaToJulianDelta("P1Y", "-1-01-01T00:00:00.000", '+', 365, 0);
	assertTimeDeltaToJulianDelta("P1Y", "-1-01-01T00:00:00.000", '+', 365, 0);
	assertTimeDeltaToJulianDelta("P1Y", "-1-02-28T00:00:00.000", '+', 365, 0);
	assertTimeDeltaToJulianDelta("P1Y", "-1-03-01T00:00:00.000", '+', 366, 0);
	assertTimeDeltaToJulianDelta("P1Y", "-1-12-31T00:00:00.000", '+', 366, 0);
	assertTimeDeltaToJulianDelta("P1Y",  "0-01-01T00:00:00.000", '+', 366, 0);
	assertTimeDeltaToJulianDelta("P1Y",  "0-02-28T00:00:00.000", '+', 366, 0);
	assertTimeDeltaToJulianDelta("P1Y",  "0-02-29T00:00:00.000", '+', 366, 0);
	assertTimeDeltaToJulianDelta("P1Y",  "0-03-01T00:00:00.000", '+', 365, 0);
	assertTimeDeltaToJulianDelta("P1Y",  "0-12-31T00:00:00.000", '+', 365, 0);
	assertTimeDeltaToJulianDelta("P1Y",  "1-01-01T00:00:00.000", '+', 365, 0);
	assertTimeDeltaToJulianDelta("P1Y",  "1-01-01T00:00:00.000", '+', 365, 0);
	assertTimeDeltaToJulianDelta("P1Y",  "1-02-28T00:00:00.000", '+', 365, 0);
	assertTimeDeltaToJulianDelta("P1Y",  "1-03-01T00:00:00.000", '+', 365, 0);
	assertTimeDeltaToJulianDelta("P1Y",  "1-12-31T00:00:00.000", '+', 365, 0);

	assertTimeDeltaToJulianDelta("-P1Y", "-1-01-01T00:00:00.000", '-', -365, 0);
	assertTimeDeltaToJulianDelta("-P1Y", "-1-01-01T00:00:00.000", '-', -365, 0);
	assertTimeDeltaToJulianDelta("-P1Y", "-1-02-28T00:00:00.000", '-', -365, 0);
	assertTimeDeltaToJulianDelta("-P1Y", "-1-03-01T00:00:00.000", '-', -365, 0);
	assertTimeDeltaToJulianDelta("-P1Y", "-1-12-31T00:00:00.000", '-', -365, 0);
	assertTimeDeltaToJulianDelta("-P1Y",  "0-01-01T00:00:00.000", '-', -365, 0);
	assertTimeDeltaToJulianDelta("-P1Y",  "0-02-28T00:00:00.000", '-', -365, 0);
	assertTimeDeltaToJulianDelta("-P1Y",  "0-02-29T00:00:00.000", '-', -365, 0);
	assertTimeDeltaToJulianDelta("-P1Y",  "0-03-01T00:00:00.000", '-', -366, 0);
	assertTimeDeltaToJulianDelta("-P1Y",  "0-12-31T00:00:00.000", '-', -366, 0);
	assertTimeDeltaToJulianDelta("-P1Y",  "1-01-01T00:00:00.000", '-', -366, 0);
	assertTimeDeltaToJulianDelta("-P1Y",  "1-01-01T00:00:00.000", '-', -366, 0);
	assertTimeDeltaToJulianDelta("-P1Y",  "1-02-28T00:00:00.000", '-', -366, 0);
	assertTimeDeltaToJulianDelta("-P1Y",  "1-03-01T00:00:00.000", '-', -365, 0);
	assertTimeDeltaToJulianDelta("-P1Y",  "1-12-31T00:00:00.000", '-', -365, 0);

	assertTimeDeltaToJulianDelta("P2Y",  "0-01-01T00:00:00.000", '+', 731, 0);
	assertTimeDeltaToJulianDelta("P6Y",  "0-01-01T00:00:00.000", '+', 2192, 0);
	assertTimeDeltaToJulianDelta("P26Y",  "0-01-01T00:00:00.000", '+', 9497, 0);
	assertTimeDeltaToJulianDelta("P126Y",  "0-01-01T00:00:00.000", '+', 46021, 0);
	assertTimeDeltaToJulianDelta("P406Y",  "0-01-01T00:00:00.000", '+', 148289, 0);
	assertTimeDeltaToJulianDelta("P10406Y",  "0-01-01T00:00:00.000", '+', 3800714, 0);
	assertTimeDeltaToJulianDelta("P10407Y",  "0-01-01T00:00:00.000", '+', 3801079, 0);
	assertTimeDeltaToJulianDelta("P10408Y",  "0-01-01T00:00:00.000", '+', 3801444, 0);
	assertTimeDeltaToJulianDelta("P10409Y",  "0-01-01T00:00:00.000", '+', 3801810, 0);
	assertTimeDeltaToJulianDelta("P10409Y",  "0-03-01T00:00:00.000", '+', 3801809, 0);
	assertTimeDeltaToJulianDelta("P10409Y",  "1-01-01T00:00:00.000", '+', 3801809, 0);
	assertTimeDeltaToJulianDelta("-P10409Y",  "1-01-01T00:00:00.000", '-', -3801810, 0);
	assertTimeDeltaToJulianDelta("-P10409Y",  "1201-01-01T00:00:00.000", '-', -3801810, 0);
	assertTimeDeltaToJulianDelta("-P10409Y",  "-1199-01-01T00:00:00.000", '-', -3801810, 0);
	assertTimeDeltaToJulianDelta("-P10409Y",  "0-01-01T00:00:00.000", '-', -3801809, 0);
	assertTimeDeltaToJulianDeltaWithoutExpactations("-P2147483648Y",  "0-01-01T00:00:00.000", '-');
	assertTimeDeltaToJulianDeltaWithoutExpactations("P2147483647Y",  "0-01-01T00:00:00.000", '+');
	assertTimeDeltaToJulianDeltaWithoutExpactations("-P2147483648Y",  "2147483647-01-01T00:00:00.000", '-');
	assertTimeDeltaToJulianDeltaWithoutExpactations("P2147483647Y",  "-2147483648-01-01T00:00:00.000", '+');

	assertTimeDeltaToJulianDelta("-P1Y",  "2147483647-01-01T00:00:00.000", '-', -365, 0);
	assertTimeDeltaToJulianDelta("P1Y",  "-2147483648-01-01T00:00:00.000", '+', 366, 0);
}
END_TEST

START_TEST(test_timeDeltaToJulianDelta_ProlepticGregorian_2)
{
	assertTimeDeltaToJulianDelta("P01M", "1-01-01T00:00:00.000", '+', 31, 0);
	assertTimeDeltaToJulianDelta("P01M", "1-02-01T00:00:00.000", '+', 28, 0);
	assertTimeDeltaToJulianDelta("P01M", "0-02-01T00:00:00.000", '+', 29, 0);
	assertTimeDeltaToJulianDelta("P01M", "0-03-01T00:00:00.000", '+', 31, 0);
	assertTimeDeltaToJulianDelta("P01M", "1-04-01T00:00:00.000", '+', 30, 0);
	assertTimeDeltaToJulianDelta("P01M", "1-05-01T00:00:00.000", '+', 31, 0);
	assertTimeDeltaToJulianDelta("P01M", "1-06-01T00:00:00.000", '+', 30, 0);
	assertTimeDeltaToJulianDelta("P01M", "1-07-01T00:00:00.000", '+', 31, 0);
	assertTimeDeltaToJulianDelta("P01M", "1-08-01T00:00:00.000", '+', 31, 0);
	assertTimeDeltaToJulianDelta("P01M", "1-09-01T00:00:00.000", '+', 30, 0);
	assertTimeDeltaToJulianDelta("P01M", "1-10-01T00:00:00.000", '+', 31, 0);
	assertTimeDeltaToJulianDelta("P01M", "1-11-01T00:00:00.000", '+', 30, 0);
	assertTimeDeltaToJulianDelta("P01M", "1-12-01T00:00:00.000", '+', 31, 0);

	assertTimeDeltaToJulianDelta("-P01M", "1-01-01T00:00:00.000", '-', -31, 0);
	assertTimeDeltaToJulianDelta("-P01M", "1-02-01T00:00:00.000", '-', -31, 0);
	assertTimeDeltaToJulianDelta("-P01M", "1-03-01T00:00:00.000", '-', -28, 0);
	assertTimeDeltaToJulianDelta("-P01M", "0-03-01T00:00:00.000", '-', -29, 0);
	assertTimeDeltaToJulianDelta("-P01M", "1-04-01T00:00:00.000", '-', -31, 0);
	assertTimeDeltaToJulianDelta("-P01M", "1-05-01T00:00:00.000", '-', -30, 0);
	assertTimeDeltaToJulianDelta("-P01M", "1-06-01T00:00:00.000", '-', -31, 0);
	assertTimeDeltaToJulianDelta("-P01M", "1-07-01T00:00:00.000", '-', -30, 0);
	assertTimeDeltaToJulianDelta("-P01M", "1-08-01T00:00:00.000", '-', -31, 0);
	assertTimeDeltaToJulianDelta("-P01M", "1-09-01T00:00:00.000", '-', -31, 0);
	assertTimeDeltaToJulianDelta("-P01M", "1-10-01T00:00:00.000", '-', -30, 0);
	assertTimeDeltaToJulianDelta("-P01M", "1-11-01T00:00:00.000", '-', -31, 0);
	assertTimeDeltaToJulianDelta("-P01M", "1-12-01T00:00:00.000", '-', -30, 0);
}
END_TEST

START_TEST(test_timeDeltaToJulianDelta_ProlepticGregorian_3)
{
	assertTimeDeltaToJulianDelta("P01D", "1-01-01T00:00:00.000", '+', 1, 0);
	assertTimeDeltaToJulianDelta("PT01H", "1-01-01T00:00:00.000", '+', 0, 3600000);
	assertTimeDeltaToJulianDelta("PT01M", "1-01-01T00:00:00.000", '+', 0, 60000);
	assertTimeDeltaToJulianDelta("PT01S", "1-01-01T00:00:00.000", '+', 0, 1000);
	assertTimeDeltaToJulianDelta("PT00.001S", "1-01-01T00:00:00.000", '+', 0, 1);

	assertTimeDeltaToJulianDelta("-P01D", "1-01-01T00:00:00.000", '-', -1, 0);
	assertTimeDeltaToJulianDelta("-PT01H", "1-01-01T00:00:00.000", '-', 0, -3600000);
	assertTimeDeltaToJulianDelta("-PT01M", "1-01-01T00:00:00.000", '-', 0, -60000);
	assertTimeDeltaToJulianDelta("-PT01S", "1-01-01T00:00:00.000", '-', 0, -1000);
	assertTimeDeltaToJulianDelta("-PT00.001S", "1-01-01T00:00:00.000", '-', 0, -1);

	assertTimeDeltaToJulianDelta("P25Y", "1-01-01T00:00:00.000", '+', 9131, 0);
	assertTimeDeltaToJulianDelta("P25Y4M", "1-01-01T00:00:00.000", '+', 9251, 0);
	assertTimeDeltaToJulianDelta("P25Y4M12D", "1-01-01T00:00:00.000", '+', 9263, 0);
	assertTimeDeltaToJulianDelta("P25Y4M12DT06H23M", "1-01-01T00:00:00.000", '+', 9263, 22980000);
	assertTimeDeltaToJulianDelta("P25Y4M12DT06H23M56S", "1-01-01T00:00:00.000", '+', 9263, 23036000);
	assertTimeDeltaToJulianDelta("P25Y4M12DT06H23M56.132S", "0-01-01T00:00:00.000", '+', 9264, 23036132);
	assertTimeDeltaToJulianDelta("P25Y4M12DT06H23M56.132S", "0-03-01T00:00:00.000", '+', 9265, 23036132);
	assertTimeDeltaToJulianDelta("-P25Y4M12DT06H23M56.132S", "0-03-01T00:00:00.000", '-', -9264, -23036132);

	assertTimeDeltaToJulianDelta("P366D", "0-01-01T00:00:00.000", '+', 366, 0);
	assertTimeDeltaToJulianDelta("P366D", "0-03-01T00:00:00.000", '+', 366, 0);
	assertTimeDeltaToJulianDelta("P366D", "1-01-01T00:00:00.000", '+', 366, 0);

	assertTimeDeltaToJulianDelta("-P366D", "0-01-01T00:00:00.000", '-', -366, 0);
	assertTimeDeltaToJulianDelta("-P366D", "0-03-01T00:00:00.000", '-', -366, 0);
	assertTimeDeltaToJulianDelta("-P366D", "1-01-01T00:00:00.000", '-', -366, 0);

	assertTimeDeltaToJulianDelta("PT24H", "1-01-01T00:00:00.000", '+', 1, 0);
	assertTimeDeltaToJulianDelta("PT30H", "1-01-01T00:00:00.000", '+', 1, 21600000);
	assertTimeDeltaToJulianDelta("PT1440M", "1-01-01T00:00:00.000", '+', 1, 0);
	assertTimeDeltaToJulianDelta("PT4335M", "1-01-01T00:00:00.000", '+', 3, 900000);
	assertTimeDeltaToJulianDelta("PT86400S", "1-01-01T00:00:00.000", '+', 1, 0);
	assertTimeDeltaToJulianDelta("PT3653389.498S", "1-01-01T00:00:00.000", '+', 42, 24589498);

	assertTimeDeltaToJulianDelta("-PT24H", "1-01-01T00:00:00.000", '-', -1, 0);
	assertTimeDeltaToJulianDelta("-PT30H", "1-01-01T00:00:00.000", '-', -1, -21600000);
	assertTimeDeltaToJulianDelta("-PT1440M", "1-01-01T00:00:00.000", '-', -1, 0);
	assertTimeDeltaToJulianDelta("-PT4335M", "1-01-01T00:00:00.000", '-', -3, -900000);
	assertTimeDeltaToJulianDelta("-PT86400S", "1-01-01T00:00:00.000", '-', -1, 0);
	assertTimeDeltaToJulianDelta("-PT3653389.498S", "1-01-01T00:00:00.000", '-', -42, -24589498);
}
END_TEST

START_TEST(test_timeDeltaToJulianDelta_YearOf365Days)
{
	assertTimeDeltaToJulianDelta("P0Y", "0-01-01T00:00:00.000", '+', 0, 0);
	assertTimeDeltaToJulianDelta("P1Y", "0-01-01T00:00:00.000", '+', 365, 0);
	assertTimeDeltaToJulianDelta("P1Y", "1-01-01T00:00:00.000", '+', 365, 0);
	assertTimeDeltaToJulianDelta("-P1Y", "0-01-01T00:00:00.000", '-', -365, 0);
	assertTimeDeltaToJulianDelta("-P1Y", "1-01-01T00:00:00.000", '-', -365, 0);

	assertTimeDeltaToJulianDelta("P01M", "1-01-01T00:00:00.000", '+', 31, 0);
	assertTimeDeltaToJulianDelta("P01M", "1-02-01T00:00:00.000", '+', 28, 0);
	assertTimeDeltaToJulianDelta("P01M", "0-02-01T00:00:00.000", '+', 28, 0);
	assertTimeDeltaToJulianDelta("P01M", "0-03-01T00:00:00.000", '+', 31, 0);
	assertTimeDeltaToJulianDelta("P01M", "1-04-01T00:00:00.000", '+', 30, 0);
	assertTimeDeltaToJulianDelta("P01M", "1-05-01T00:00:00.000", '+', 31, 0);
	assertTimeDeltaToJulianDelta("P01M", "1-06-01T00:00:00.000", '+', 30, 0);
	assertTimeDeltaToJulianDelta("P01M", "1-07-01T00:00:00.000", '+', 31, 0);
	assertTimeDeltaToJulianDelta("P01M", "1-08-01T00:00:00.000", '+', 31, 0);
	assertTimeDeltaToJulianDelta("P01M", "1-09-01T00:00:00.000", '+', 30, 0);
	assertTimeDeltaToJulianDelta("P01M", "1-10-01T00:00:00.000", '+', 31, 0);
	assertTimeDeltaToJulianDelta("P01M", "1-11-01T00:00:00.000", '+', 30, 0);
	assertTimeDeltaToJulianDelta("P01M", "1-12-01T00:00:00.000", '+', 31, 0);

	assertTimeDeltaToJulianDelta("-P01M", "1-01-01T00:00:00.000", '-', -31, 0);
	assertTimeDeltaToJulianDelta("-P01M", "1-02-01T00:00:00.000", '-', -31, 0);
	assertTimeDeltaToJulianDelta("-P01M", "1-03-01T00:00:00.000", '-', -28, 0);
	assertTimeDeltaToJulianDelta("-P01M", "0-03-01T00:00:00.000", '-', -28, 0);
	assertTimeDeltaToJulianDelta("-P01M", "1-04-01T00:00:00.000", '-', -31, 0);
	assertTimeDeltaToJulianDelta("-P01M", "1-05-01T00:00:00.000", '-', -30, 0);
	assertTimeDeltaToJulianDelta("-P01M", "1-06-01T00:00:00.000", '-', -31, 0);
	assertTimeDeltaToJulianDelta("-P01M", "1-07-01T00:00:00.000", '-', -30, 0);
	assertTimeDeltaToJulianDelta("-P01M", "1-08-01T00:00:00.000", '-', -31, 0);
	assertTimeDeltaToJulianDelta("-P01M", "1-09-01T00:00:00.000", '-', -31, 0);
	assertTimeDeltaToJulianDelta("-P01M", "1-10-01T00:00:00.000", '-', -30, 0);
	assertTimeDeltaToJulianDelta("-P01M", "1-11-01T00:00:00.000", '-', -31, 0);
	assertTimeDeltaToJulianDelta("-P01M", "1-12-01T00:00:00.000", '-', -30, 0);

}
END_TEST

START_TEST(test_timeDeltaToJulianDelta_YearOf360Days)
{
	assertTimeDeltaToJulianDelta("P0Y", "0-01-01T00:00:00.000", '+', 0, 0);
	assertTimeDeltaToJulianDelta("P1Y", "0-01-01T00:00:00.000", '+', 360, 0);
	assertTimeDeltaToJulianDelta("P1Y", "1-01-01T00:00:00.000", '+', 360, 0);
	assertTimeDeltaToJulianDelta("-P1Y", "0-01-01T00:00:00.000", '-', -360, 0);
	assertTimeDeltaToJulianDelta("-P1Y", "1-01-01T00:00:00.000", '-', -360, 0);

	assertTimeDeltaToJulianDelta("P01M", "1-01-01T00:00:00.000", '+', 30, 0);
	assertTimeDeltaToJulianDelta("P01M", "1-02-01T00:00:00.000", '+', 30, 0);
	assertTimeDeltaToJulianDelta("P01M", "0-02-01T00:00:00.000", '+', 30, 0);
	assertTimeDeltaToJulianDelta("P01M", "0-03-01T00:00:00.000", '+', 30, 0);
	assertTimeDeltaToJulianDelta("P01M", "1-04-01T00:00:00.000", '+', 30, 0);
	assertTimeDeltaToJulianDelta("P01M", "1-05-01T00:00:00.000", '+', 30, 0);
	assertTimeDeltaToJulianDelta("P01M", "1-06-01T00:00:00.000", '+', 30, 0);
	assertTimeDeltaToJulianDelta("P01M", "1-07-01T00:00:00.000", '+', 30, 0);
	assertTimeDeltaToJulianDelta("P01M", "1-08-01T00:00:00.000", '+', 30, 0);
	assertTimeDeltaToJulianDelta("P01M", "1-09-01T00:00:00.000", '+', 30, 0);
	assertTimeDeltaToJulianDelta("P01M", "1-10-01T00:00:00.000", '+', 30, 0);
	assertTimeDeltaToJulianDelta("P01M", "1-11-01T00:00:00.000", '+', 30, 0);
	assertTimeDeltaToJulianDelta("P01M", "1-12-01T00:00:00.000", '+', 30, 0);

	assertTimeDeltaToJulianDelta("-P01M", "1-01-01T00:00:00.000", '-', -30, 0);
	assertTimeDeltaToJulianDelta("-P01M", "1-02-01T00:00:00.000", '-', -30, 0);
	assertTimeDeltaToJulianDelta("-P01M", "1-03-01T00:00:00.000", '-', -30, 0);
	assertTimeDeltaToJulianDelta("-P01M", "0-03-01T00:00:00.000", '-', -30, 0);
	assertTimeDeltaToJulianDelta("-P01M", "1-04-01T00:00:00.000", '-', -30, 0);
	assertTimeDeltaToJulianDelta("-P01M", "1-05-01T00:00:00.000", '-', -30, 0);
	assertTimeDeltaToJulianDelta("-P01M", "1-06-01T00:00:00.000", '-', -30, 0);
	assertTimeDeltaToJulianDelta("-P01M", "1-07-01T00:00:00.000", '-', -30, 0);
	assertTimeDeltaToJulianDelta("-P01M", "1-08-01T00:00:00.000", '-', -30, 0);
	assertTimeDeltaToJulianDelta("-P01M", "1-09-01T00:00:00.000", '-', -30, 0);
	assertTimeDeltaToJulianDelta("-P01M", "1-10-01T00:00:00.000", '-', -30, 0);
	assertTimeDeltaToJulianDelta("-P01M", "1-11-01T00:00:00.000", '-', -30, 0);
	assertTimeDeltaToJulianDelta("-P01M", "1-12-01T00:00:00.000", '-', -30, 0);
}
END_TEST

START_TEST(test_timeDeltaToJulianDelta_highDeltas)
{
	assertTimeDeltaToJulianDeltaWithoutExpactations("P0Y",  "0-01-01T00:00:00.000", '+');
	assertTimeDeltaToJulianDeltaWithoutExpactations("P1Y",  "0-01-01T00:00:00.000", '+');
	assertTimeDeltaToJulianDeltaWithoutExpactations("P9Y",  "0-01-01T00:00:00.000", '+');
	assertTimeDeltaToJulianDeltaWithoutExpactations("P99Y",  "0-01-01T00:00:00.000", '+');
	assertTimeDeltaToJulianDeltaWithoutExpactations("P999Y",  "0-01-01T00:00:00.000", '+');
	assertTimeDeltaToJulianDeltaWithoutExpactations("P9999Y",  "0-01-01T00:00:00.000", '+');
	assertTimeDeltaToJulianDeltaWithoutExpactations("P99999Y",  "0-01-01T00:00:00.000", '+');
	assertTimeDeltaToJulianDeltaWithoutExpactations("P999999Y",  "0-01-01T00:00:00.000", '+');
	assertTimeDeltaToJulianDeltaWithoutExpactations("P9999999Y",  "0-01-01T00:00:00.000", '+');
	assertTimeDeltaToJulianDeltaWithoutExpactations("P99999999Y",  "0-01-01T00:00:00.000", '+');
	assertTimeDeltaToJulianDeltaWithoutExpactations("P999999999Y",  "0-01-01T00:00:00.000", '+');
	assertTimeDeltaToJulianDeltaWithoutExpactations("P2147483647Y",  "0-01-01T00:00:00.000", '+');
	assertTimeDeltaToJulianDeltaWithoutExpactations("-P2147483647Y",  "0-01-01T00:00:00.000", '-');
	assertTimeDeltaToJulianDeltaWithoutExpactations("-P2147483648Y",  "0-01-01T00:00:00.000", '-');
}
END_TEST

START_TEST(test_julianDeltaToTimeDelta)
{
	fail("Test not yet implemented");
}
END_TEST

START_TEST(test_divideTimeDeltaInSeconds)
{
	assertDivideTimeDeltaInSeconds("PT00.001S", "PT00.001S", 1, 0);
	assertDivideTimeDeltaInSeconds("PT00.999S", "PT00.001S", 999, 0);
	assertDivideTimeDeltaInSeconds("PT00.999S", "PT00.500S", 1, 499);
	assertDivideTimeDeltaInSeconds("PT01.000S", "PT00.001S", 1000, 0);

	assertDivideTimeDeltaInSeconds("PT01.000S", "PT01.000S", 1, 0);
	assertDivideTimeDeltaInSeconds("PT01.001S", "PT01.000S", 1, 1);
	assertDivideTimeDeltaInSeconds("PT01.000S", "PT01.001S", 0, 1000);
	assertDivideTimeDeltaInSeconds("PT01.999S", "PT01.000S", 1, 999);
	assertDivideTimeDeltaInSeconds("PT01.000S", "PT01.999S", 0, 1000);
	assertDivideTimeDeltaInSeconds("PT02.000S", "PT01.000S", 2, 0);
	assertDivideTimeDeltaInSeconds("PT01.000S", "PT02.000S", 0, 1000);
	assertDivideTimeDeltaInSeconds("PT59.999S", "PT01.000S", 59, 999);
	assertDivideTimeDeltaInSeconds("PT01.000S", "PT59.999S", 0, 1000);
	assertDivideTimeDeltaInSeconds("PT59.998S", "PT59.999S", 0, 59998);
//	assertDivideTimeDeltaInSeconds("PT2147483647.999S", "PT01.000S", 2147483647, 999); //FAILS
//	assertDivideTimeDeltaInSeconds("PT01.000S", "PT2147483647.999S", 0, 2147483647999); //FAILS

	assertDivideTimeDeltaInSeconds("PT1000.234S", "PT45.879S", 21, 36775);
	assertDivideTimeDeltaInSeconds("PT45.879S", "PT1000.234S", 0, 45879);

	assertDivideTimeDeltaInSeconds("PT01M", "PT01.000S", 60, 0);
	assertDivideTimeDeltaInSeconds("PT01M", "PT00.001S", 60000, 0);

	assertDivideTimeDeltaInSeconds("PT01H", "PT01M", 60, 0);
	assertDivideTimeDeltaInSeconds("PT01H", "PT01.000S", 3600, 0);
	assertDivideTimeDeltaInSeconds("PT01H", "PT00.001S", 3600000, 0);

	assertDivideTimeDeltaInSeconds("P01D", "PT01H", 24, 0);
	assertDivideTimeDeltaInSeconds("P01D", "PT01M", 1440, 0);
	assertDivideTimeDeltaInSeconds("P01D", "PT01S", 86400, 0);
	assertDivideTimeDeltaInSeconds("P01D", "PT00.001S", 86400000, 0);
}
END_TEST

START_TEST(test_getTimeDeltaFromDate_ProlepticGregorian)
{
	assertGetTimeDeltaFromDate("-2147483648-01-01", "-2147483648-01-01", "PT00.000S");
	assertGetTimeDeltaFromDate("0-01-01", "0-01-01", "PT00.000S");
	assertGetTimeDeltaFromDate("2147483647-12-31", "2147483647-12-31", "PT00.000S");

	assertGetTimeDeltaFromDate("0-01-02", "0-01-01", "P01D");
	assertGetTimeDeltaFromDate("0-02-01", "0-01-01", "P01M");
	assertGetTimeDeltaFromDate("1-01-01", "0-01-01", "P1Y");
	assertGetTimeDeltaFromDate("2-01-01", "1-01-01", "P1Y");
	assertGetTimeDeltaFromDate("0-01-01", "0-01-02", "-P01D");
	assertGetTimeDeltaFromDate("0-01-01", "0-02-01", "-P01M");
	assertGetTimeDeltaFromDate("0-01-01", "1-01-01", "-P1Y");
	assertGetTimeDeltaFromDate("1-01-01", "2-01-01", "-P1Y");
	assertGetTimeDeltaFromDate("1-01-01", "2-12-01", "-P1Y11M");
	assertGetTimeDeltaFromDate("1-01-01", "2-12-31", "-P1Y11M30D");

	assertGetTimeDeltaFromDate("-1-12-31", "0-01-01", "-P01D");
	assertGetTimeDeltaFromDate("-1-12-01", "0-01-01", "-P01M");
	assertGetTimeDeltaFromDate("-1-01-01", "0-01-01", "-P1Y");
	assertGetTimeDeltaFromDate("-2-01-01", "-1-01-01", "-P1Y");
	assertGetTimeDeltaFromDate("0-01-01", "-1-12-31", "P01D");
	assertGetTimeDeltaFromDate("0-01-01", "-1-12-01", "P01M");
	assertGetTimeDeltaFromDate("0-01-01", "-1-01-01", "P1Y");
	assertGetTimeDeltaFromDate("-1-01-01", "-2-01-01", "P1Y");

	assertGetTimeDeltaFromDate("2014-03-12", "2014-03-12", "PT00.000S");
	assertGetTimeDeltaFromDate("2014-03-12", "2014-03-11", "P01D");
	assertGetTimeDeltaFromDate("2014-03-12", "2014-03-01", "P11D");
	assertGetTimeDeltaFromDate("2014-03-12", "2014-03-01", "P11D");
	assertGetTimeDeltaFromDate("2014-03-12", "2014-02-28", "P12D");
	assertGetTimeDeltaFromDate("2014-03-12", "2014-02-13", "P27D");
	assertGetTimeDeltaFromDate("2014-03-12", "2014-02-12", "P01M");
	assertGetTimeDeltaFromDate("2014-03-31", "2014-02-28", "P01M03D");
	assertGetTimeDeltaFromDate("2014-02-28", "2014-01-31", "P28D");
	assertGetTimeDeltaFromDate("2014-03-12", "2014-02-01", "P01M11D");
	assertGetTimeDeltaFromDate("2014-03-12", "2014-01-25", "P01M18D");
	assertGetTimeDeltaFromDate("2014-03-12", "2013-03-12", "P1Y");
	assertGetTimeDeltaFromDate("2014-03-12", "2013-03-11", "P1Y01D");
	assertGetTimeDeltaFromDate("2014-03-12", "2013-03-05", "P1Y07D");
	assertGetTimeDeltaFromDate("2014-03-12", "2013-03-01", "P1Y11D");
	assertGetTimeDeltaFromDate("2014-03-12", "2013-02-28", "P1Y12D");
	assertGetTimeDeltaFromDate("2014-03-12", "2013-02-13", "P1Y27D");
	assertGetTimeDeltaFromDate("2014-03-12", "2013-02-12", "P1Y01M");
	assertGetTimeDeltaFromDate("2014-03-12", "2013-02-07", "P1Y01M05D");
	assertGetTimeDeltaFromDate("2014-03-12", "2013-01-30", "P1Y01M13D");
	assertGetTimeDeltaFromDate("2014-03-12", "2012-09-25", "P1Y05M17D");

	assertGetTimeDeltaFromDate("2014-03-12", "2014-03-12", "PT00.000S");
	assertGetTimeDeltaFromDate("2014-03-11", "2014-03-12", "-P01D");
	assertGetTimeDeltaFromDate("2014-03-01", "2014-03-12", "-P11D");
	assertGetTimeDeltaFromDate("2014-03-01", "2014-03-12", "-P11D");
	assertGetTimeDeltaFromDate("2014-02-28", "2014-03-12", "-P12D");
	assertGetTimeDeltaFromDate("2014-02-13", "2014-03-12", "-P27D");
	assertGetTimeDeltaFromDate("2014-02-12", "2014-03-12", "-P01M");
	assertGetTimeDeltaFromDate("2014-02-28", "2014-03-31", "-P01M03D");
	assertGetTimeDeltaFromDate("2014-01-31", "2014-02-28", "-P28D");
	assertGetTimeDeltaFromDate("2014-02-01", "2014-03-12", "-P01M11D");
	assertGetTimeDeltaFromDate("2014-01-25", "2014-03-12", "-P01M15D");
	assertGetTimeDeltaFromDate("2013-03-12", "2014-03-12", "-P1Y");
	assertGetTimeDeltaFromDate("2013-03-11", "2014-03-12", "-P1Y01D");
	assertGetTimeDeltaFromDate("2013-03-05", "2014-03-12", "-P1Y07D");
	assertGetTimeDeltaFromDate("2013-03-01", "2014-03-12", "-P1Y11D");
	assertGetTimeDeltaFromDate("2013-02-28", "2014-03-12", "-P1Y12D");
	assertGetTimeDeltaFromDate("2013-02-13", "2014-03-12", "-P1Y27D");
	assertGetTimeDeltaFromDate("2013-02-12", "2014-03-12", "-P1Y01M");
	assertGetTimeDeltaFromDate("2013-02-07", "2014-03-12", "-P1Y01M05D");
	assertGetTimeDeltaFromDate("2013-01-30", "2014-03-12", "-P1Y01M10D");
	assertGetTimeDeltaFromDate("2012-09-25", "2014-03-12", "-P1Y05M15D");

	assertGetTimeDeltaFromDate("-2014-03-12", "-2014-03-12", "PT00.000S");
	assertGetTimeDeltaFromDate("-2014-03-12", "-2014-03-11", "P01D");
	assertGetTimeDeltaFromDate("-2014-03-12", "-2014-03-01", "P11D");
	assertGetTimeDeltaFromDate("-2014-03-12", "-2014-03-01", "P11D");
	assertGetTimeDeltaFromDate("-2014-03-12", "-2014-02-28", "P12D");
	assertGetTimeDeltaFromDate("-2014-03-12", "-2014-02-13", "P27D");
	assertGetTimeDeltaFromDate("-2014-03-12", "-2014-02-12", "P01M");
	assertGetTimeDeltaFromDate("-2014-03-31", "-2014-02-28", "P01M03D");
	assertGetTimeDeltaFromDate("-2014-02-28", "-2014-01-31", "P28D");
	assertGetTimeDeltaFromDate("-2014-03-12", "-2014-02-01", "P01M11D");
	assertGetTimeDeltaFromDate("-2014-03-12", "-2014-01-25", "P01M18D");
	assertGetTimeDeltaFromDate("-2014-03-12", "-2015-03-12", "P1Y");
	assertGetTimeDeltaFromDate("-2014-03-12", "-2015-03-11", "P1Y01D");
	assertGetTimeDeltaFromDate("-2014-03-12", "-2015-03-05", "P1Y07D");
	assertGetTimeDeltaFromDate("-2014-03-12", "-2015-03-01", "P1Y11D");
	assertGetTimeDeltaFromDate("-2014-03-12", "-2015-02-28", "P1Y12D");
	assertGetTimeDeltaFromDate("-2014-03-12", "-2015-02-13", "P1Y27D");
	assertGetTimeDeltaFromDate("-2014-03-12", "-2015-02-12", "P1Y01M");
	assertGetTimeDeltaFromDate("-2014-03-12", "-2015-02-07", "P1Y01M05D");
	assertGetTimeDeltaFromDate("-2014-03-12", "-2015-01-30", "P1Y01M13D");
	assertGetTimeDeltaFromDate("-2014-03-12", "-2016-09-25", "P1Y05M17D");

	assertGetTimeDeltaFromDate("-2014-03-12", "-2014-03-12", "PT00.000S");
	assertGetTimeDeltaFromDate("-2014-03-11", "-2014-03-12", "-P01D");
	assertGetTimeDeltaFromDate("-2014-03-01", "-2014-03-12", "-P11D");
	assertGetTimeDeltaFromDate("-2014-03-01", "-2014-03-12", "-P11D");
	assertGetTimeDeltaFromDate("-2014-02-28", "-2014-03-12", "-P12D");
	assertGetTimeDeltaFromDate("-2014-02-13", "-2014-03-12", "-P27D");
	assertGetTimeDeltaFromDate("-2014-02-12", "-2014-03-12", "-P01M");
	assertGetTimeDeltaFromDate("-2014-02-28", "-2014-03-31", "-P01M03D");
	assertGetTimeDeltaFromDate("-2014-01-31", "-2014-02-28", "-P28D");
	assertGetTimeDeltaFromDate("-2014-02-01", "-2014-03-12", "-P01M11D");
	assertGetTimeDeltaFromDate("-2014-01-25", "-2014-03-12", "-P01M15D");
	assertGetTimeDeltaFromDate("-2015-03-12", "-2014-03-12", "-P1Y");
	assertGetTimeDeltaFromDate("-2015-03-11", "-2014-03-12", "-P1Y01D");
	assertGetTimeDeltaFromDate("-2015-03-05", "-2014-03-12", "-P1Y07D");
	assertGetTimeDeltaFromDate("-2015-03-01", "-2014-03-12", "-P1Y11D");
	assertGetTimeDeltaFromDate("-2015-02-28", "-2014-03-12", "-P1Y12D");
	assertGetTimeDeltaFromDate("-2015-02-13", "-2014-03-12", "-P1Y27D");
	assertGetTimeDeltaFromDate("-2015-02-12", "-2014-03-12", "-P1Y01M");
	assertGetTimeDeltaFromDate("-2015-02-07", "-2014-03-12", "-P1Y01M05D");
	assertGetTimeDeltaFromDate("-2015-01-30", "-2014-03-12", "-P1Y01M10D");
	assertGetTimeDeltaFromDate("-2016-09-25", "-2014-03-12", "-P1Y05M15D");
}
END_TEST

START_TEST(test_getTimeDeltaFromDate_ProlepticGregorian_LeapYears)
{
	assertGetTimeDeltaFromDate("2012-03-12", "2012-03-12", "PT00.000S");
	assertGetTimeDeltaFromDate("2012-03-12", "2012-03-11", "P01D");
	assertGetTimeDeltaFromDate("2012-03-12", "2012-03-01", "P11D");
	assertGetTimeDeltaFromDate("2012-03-12", "2012-02-29", "P12D");
	assertGetTimeDeltaFromDate("2012-03-12", "2012-02-28", "P13D");
	assertGetTimeDeltaFromDate("2012-03-12", "2012-02-13", "P28D");
	assertGetTimeDeltaFromDate("2012-03-12", "2012-02-12", "P01M");
	assertGetTimeDeltaFromDate("2012-03-31", "2012-02-28", "P01M03D");
	assertGetTimeDeltaFromDate("2012-03-31", "2012-02-29", "P01M02D");
	assertGetTimeDeltaFromDate("2012-02-28", "2012-01-31", "P28D");
	assertGetTimeDeltaFromDate("2012-02-29", "2012-01-31", "P29D");
	assertGetTimeDeltaFromDate("2012-03-12", "2012-02-01", "P01M11D");
	assertGetTimeDeltaFromDate("2012-03-12", "2012-01-25", "P01M18D");
	assertGetTimeDeltaFromDate("2012-03-12", "2011-03-12", "P1Y");
	assertGetTimeDeltaFromDate("2012-03-12", "2011-03-11", "P1Y01D");
	assertGetTimeDeltaFromDate("2012-03-12", "2011-03-05", "P1Y07D");
	assertGetTimeDeltaFromDate("2012-03-12", "2011-03-01", "P1Y11D");
	assertGetTimeDeltaFromDate("2012-03-12", "2011-02-28", "P1Y12D");
	assertGetTimeDeltaFromDate("2012-03-12", "2011-02-13", "P1Y27D");
	assertGetTimeDeltaFromDate("2012-03-12", "2011-02-12", "P1Y01M");
	assertGetTimeDeltaFromDate("2012-03-12", "2011-02-07", "P1Y01M05D");
	assertGetTimeDeltaFromDate("2012-03-12", "2011-01-30", "P1Y01M13D");
	assertGetTimeDeltaFromDate("2012-03-12", "2010-09-25", "P1Y05M17D");
	assertGetTimeDeltaFromDate("2013-03-12", "2012-03-12", "P1Y");
	assertGetTimeDeltaFromDate("2013-03-12", "2012-03-11", "P1Y01D");
	assertGetTimeDeltaFromDate("2013-03-12", "2012-03-05", "P1Y07D");
	assertGetTimeDeltaFromDate("2013-03-12", "2012-03-01", "P1Y11D");
	assertGetTimeDeltaFromDate("2013-03-12", "2012-02-29", "P1Y12D");
	assertGetTimeDeltaFromDate("2013-03-12", "2012-02-28", "P1Y13D");
	assertGetTimeDeltaFromDate("2013-03-12", "2012-02-13", "P1Y28D");
	assertGetTimeDeltaFromDate("2013-03-12", "2012-02-12", "P1Y01M");
	assertGetTimeDeltaFromDate("2013-03-12", "2012-02-07", "P1Y01M05D");
	assertGetTimeDeltaFromDate("2013-03-12", "2012-01-30", "P1Y01M13D");
	assertGetTimeDeltaFromDate("2013-03-12", "2011-09-25", "P1Y05M17D");

	assertGetTimeDeltaFromDate("2012-03-12", "2012-03-12", "PT00.000S");
	assertGetTimeDeltaFromDate("2012-03-11", "2012-03-12", "-P01D");
	assertGetTimeDeltaFromDate("2012-03-01", "2012-03-12", "-P11D");
	assertGetTimeDeltaFromDate("2012-02-29", "2012-03-12", "-P12D");
	assertGetTimeDeltaFromDate("2012-02-28", "2012-03-12", "-P13D");
	assertGetTimeDeltaFromDate("2012-02-13", "2012-03-12", "-P28D");
	assertGetTimeDeltaFromDate("2012-02-12", "2012-03-12", "-P01M");
	assertGetTimeDeltaFromDate("2012-02-28", "2012-03-31", "-P01M03D");
	assertGetTimeDeltaFromDate("2012-02-29", "2012-03-31", "-P01M02D");
	assertGetTimeDeltaFromDate("2012-01-31", "2012-02-28", "-P28D");
	assertGetTimeDeltaFromDate("2012-01-31", "2012-02-29", "-P29D");
	assertGetTimeDeltaFromDate("2012-02-01", "2012-03-12", "-P01M11D");
	assertGetTimeDeltaFromDate("2012-01-25", "2012-03-12", "-P01M16D");
	assertGetTimeDeltaFromDate("2011-03-12", "2012-03-12", "-P1Y");
	assertGetTimeDeltaFromDate("2011-03-11", "2012-03-12", "-P1Y01D");
	assertGetTimeDeltaFromDate("2011-03-05", "2012-03-12", "-P1Y07D");
	assertGetTimeDeltaFromDate("2011-03-01", "2012-03-12", "-P1Y11D");
	assertGetTimeDeltaFromDate("2011-02-28", "2012-03-12", "-P1Y13D");
	assertGetTimeDeltaFromDate("2011-02-13", "2012-03-12", "-P1Y28D");
	assertGetTimeDeltaFromDate("2011-02-12", "2012-03-12", "-P1Y01M");
	assertGetTimeDeltaFromDate("2011-02-07", "2012-03-12", "-P1Y01M05D");
	assertGetTimeDeltaFromDate("2011-01-30", "2012-03-12", "-P1Y01M11D");
	assertGetTimeDeltaFromDate("2010-09-25", "2012-03-12", "-P1Y05M16D");
	assertGetTimeDeltaFromDate("2012-03-12", "2013-03-12", "-P1Y");
	assertGetTimeDeltaFromDate("2012-03-11", "2013-03-12", "-P1Y01D");
	assertGetTimeDeltaFromDate("2012-03-05", "2013-03-12", "-P1Y07D");
	assertGetTimeDeltaFromDate("2012-03-01", "2013-03-12", "-P1Y11D");
	assertGetTimeDeltaFromDate("2012-02-29", "2013-03-12", "-P1Y11D");
	assertGetTimeDeltaFromDate("2012-02-28", "2013-03-12", "-P1Y12D");
	assertGetTimeDeltaFromDate("2012-02-13", "2013-03-12", "-P1Y27D");
	assertGetTimeDeltaFromDate("2012-02-12", "2013-03-12", "-P1Y01M");
	assertGetTimeDeltaFromDate("2012-02-07", "2013-03-12", "-P1Y01M05D");
	assertGetTimeDeltaFromDate("2012-01-30", "2013-03-12", "-P1Y01M10D");
	assertGetTimeDeltaFromDate("2011-09-25", "2013-03-12", "-P1Y05M15D");

	assertGetTimeDeltaFromDate("-2012-03-12", "-2012-03-12", "PT00.000S");
	assertGetTimeDeltaFromDate("-2012-03-12", "-2012-03-11", "P01D");
	assertGetTimeDeltaFromDate("-2012-03-12", "-2012-03-01", "P11D");
	assertGetTimeDeltaFromDate("-2012-03-12", "-2012-02-29", "P12D");
	assertGetTimeDeltaFromDate("-2012-03-12", "-2012-02-28", "P13D");
	assertGetTimeDeltaFromDate("-2012-03-12", "-2012-02-13", "P28D");
	assertGetTimeDeltaFromDate("-2012-03-12", "-2012-02-12", "P01M");
	assertGetTimeDeltaFromDate("-2012-03-31", "-2012-02-28", "P01M03D");
	assertGetTimeDeltaFromDate("-2012-03-31", "-2012-02-29", "P01M02D");
	assertGetTimeDeltaFromDate("-2012-02-28", "-2012-01-31", "P28D");
	assertGetTimeDeltaFromDate("-2012-02-29", "-2012-01-31", "P29D");
	assertGetTimeDeltaFromDate("-2012-03-12", "-2012-02-01", "P01M11D");
	assertGetTimeDeltaFromDate("-2012-03-12", "-2012-01-25", "P01M18D");
	assertGetTimeDeltaFromDate("-2012-03-12", "-2013-03-12", "P1Y");
	assertGetTimeDeltaFromDate("-2012-03-12", "-2013-03-11", "P1Y01D");
	assertGetTimeDeltaFromDate("-2012-03-12", "-2013-03-05", "P1Y07D");
	assertGetTimeDeltaFromDate("-2012-03-12", "-2013-03-01", "P1Y11D");
	assertGetTimeDeltaFromDate("-2012-03-12", "-2013-02-28", "P1Y12D");
	assertGetTimeDeltaFromDate("-2012-03-12", "-2013-02-13", "P1Y27D");
	assertGetTimeDeltaFromDate("-2012-03-12", "-2013-02-12", "P1Y01M");
	assertGetTimeDeltaFromDate("-2012-03-12", "-2013-02-07", "P1Y01M05D");
	assertGetTimeDeltaFromDate("-2012-03-12", "-2013-01-30", "P1Y01M13D");
	assertGetTimeDeltaFromDate("-2012-03-12", "-2014-09-25", "P1Y05M17D");
	assertGetTimeDeltaFromDate("-2011-03-12", "-2012-03-12", "P1Y");
	assertGetTimeDeltaFromDate("-2011-03-12", "-2012-03-11", "P1Y01D");
	assertGetTimeDeltaFromDate("-2011-03-12", "-2012-03-05", "P1Y07D");
	assertGetTimeDeltaFromDate("-2011-03-12", "-2012-03-01", "P1Y11D");
	assertGetTimeDeltaFromDate("-2011-03-12", "-2012-02-29", "P1Y12D");
	assertGetTimeDeltaFromDate("-2011-03-12", "-2012-02-28", "P1Y13D");
	assertGetTimeDeltaFromDate("-2011-03-12", "-2012-02-13", "P1Y28D");
	assertGetTimeDeltaFromDate("-2011-03-12", "-2012-02-12", "P1Y01M");
	assertGetTimeDeltaFromDate("-2011-03-12", "-2012-02-07", "P1Y01M05D");
	assertGetTimeDeltaFromDate("-2011-03-12", "-2012-01-30", "P1Y01M13D");
	assertGetTimeDeltaFromDate("-2011-03-12", "-2013-09-25", "P1Y05M17D");

	assertGetTimeDeltaFromDate("-2012-03-12", "-2012-03-12", "PT00.000S");
	assertGetTimeDeltaFromDate("-2012-03-11", "-2012-03-12", "-P01D");
	assertGetTimeDeltaFromDate("-2012-03-01", "-2012-03-12", "-P11D");
	assertGetTimeDeltaFromDate("-2012-02-29", "-2012-03-12", "-P12D");
	assertGetTimeDeltaFromDate("-2012-02-28", "-2012-03-12", "-P13D");
	assertGetTimeDeltaFromDate("-2012-02-13", "-2012-03-12", "-P28D");
	assertGetTimeDeltaFromDate("-2012-02-12", "-2012-03-12", "-P01M");
	assertGetTimeDeltaFromDate("-2012-02-28", "-2012-03-31", "-P01M03D");
	assertGetTimeDeltaFromDate("-2012-02-29", "-2012-03-31", "-P01M02D");
	assertGetTimeDeltaFromDate("-2012-01-31", "-2012-02-28", "-P28D");
	assertGetTimeDeltaFromDate("-2012-01-31", "-2012-02-29", "-P29D");
	assertGetTimeDeltaFromDate("-2012-02-01", "-2012-03-12", "-P01M11D");
	assertGetTimeDeltaFromDate("-2012-01-25", "-2012-03-12", "-P01M16D");
	assertGetTimeDeltaFromDate("-2013-03-12", "-2012-03-12", "-P1Y");
	assertGetTimeDeltaFromDate("-2013-03-11", "-2012-03-12", "-P1Y01D");
	assertGetTimeDeltaFromDate("-2013-03-05", "-2012-03-12", "-P1Y07D");
	assertGetTimeDeltaFromDate("-2013-03-01", "-2012-03-12", "-P1Y11D");
	assertGetTimeDeltaFromDate("-2013-02-28", "-2012-03-12", "-P1Y13D");
	assertGetTimeDeltaFromDate("-2013-02-13", "-2012-03-12", "-P1Y28D");
	assertGetTimeDeltaFromDate("-2013-02-12", "-2012-03-12", "-P1Y01M");
	assertGetTimeDeltaFromDate("-2013-02-07", "-2012-03-12", "-P1Y01M05D");
	assertGetTimeDeltaFromDate("-2013-01-30", "-2012-03-12", "-P1Y01M11D");
	assertGetTimeDeltaFromDate("-2014-09-25", "-2012-03-12", "-P1Y05M16D");
	assertGetTimeDeltaFromDate("-2012-03-12", "-2011-03-12", "-P1Y");
	assertGetTimeDeltaFromDate("-2012-03-11", "-2011-03-12", "-P1Y01D");
	assertGetTimeDeltaFromDate("-2012-03-05", "-2011-03-12", "-P1Y07D");
	assertGetTimeDeltaFromDate("-2012-03-01", "-2011-03-12", "-P1Y11D");
	assertGetTimeDeltaFromDate("-2012-02-29", "-2011-03-12", "-P1Y11D");
	assertGetTimeDeltaFromDate("-2012-02-28", "-2011-03-12", "-P1Y12D");
	assertGetTimeDeltaFromDate("-2012-02-13", "-2011-03-12", "-P1Y27D");
	assertGetTimeDeltaFromDate("-2012-02-12", "-2011-03-12", "-P1Y01M");
	assertGetTimeDeltaFromDate("-2012-02-07", "-2011-03-12", "-P1Y01M05D");
	assertGetTimeDeltaFromDate("-2012-01-30", "-2011-03-12", "-P1Y01M10D");
	assertGetTimeDeltaFromDate("-2013-09-25", "-2011-03-12", "-P1Y05M15D");
}
END_TEST

START_TEST(test_getTimeDeltaFromDate_ProlepticGregorian_HighDeltas)
{
	assertGetTimeDeltaFromDate("-2147483648-01-01", "-2147483648-01-01", "PT00.000S");
	assertGetTimeDeltaFromDate("-2147483648-01-01", "-2147483647-01-01", "-P1Y");
	assertGetTimeDeltaFromDate("-2147483647-01-01", "-2147483648-01-01", "P1Y");
	assertGetTimeDeltaFromDate("2147483647-01-01", "2147483647-01-01", "PT00.000S");
	assertGetTimeDeltaFromDate("2147483647-01-01", "2147483646-01-01", "P1Y");
	assertGetTimeDeltaFromDate("2147483646-01-01", "2147483647-01-01", "-P1Y");

	assertGetTimeDeltaFromDate("-83648-01-01", "-1-01-01", "-P83647Y");
	assertGetTimeDeltaFromDate("-483648-01-01", "-1-01-01", "-P483647Y");
	assertGetTimeDeltaFromDate("-5880000-01-01", "-1-01-01", "-P5879999Y");
	assertGetTimeDeltaFromDate("-5880001-02-01", "-1-01-01", "-P5879999Y11M");
	assertGetTimeDeltaFromDate("-5880001-01-02", "-1-01-01", "-P5879999Y11M30D");

	assertGetTimeDeltaFromDate("-5880001-01-01", "-1-01-01", "-P5880000Y");
	assertGetTimeDeltaFromDate("-5880000-01-01", "0-01-01", "-P5880000Y");
	assertGetTimeDeltaFromDate("-7483648-01-01", "-1-01-01", "-P7483647Y");
	assertGetTimeDeltaFromDate("-47483648-01-01", "-1-01-01", "-P47483647Y");
	assertGetTimeDeltaFromDate("-147483648-01-01", "-1-01-01", "-P147483647Y");
	assertGetTimeDeltaFromDate("-1147483648-01-01", "-1-01-01", "-P1147483647Y");
	assertGetTimeDeltaFromDate("-2147483648-01-01", "-1-01-01", "-P2147483647Y");
	assertGetTimeDeltaFromDate("-2147483648-01-01", "0-01-01", "-P2147483648Y");
	assertGetTimeDeltaFromDate("-1-01-01", "-2147483648-01-01", "P2147483647Y");
	assertGetTimeDeltaFromDate("0-01-01", "-2147483647-01-01", "P2147483647Y");

	assertGetTimeDeltaFromDate("83648-01-01", "1-01-01", "P83647Y");
	assertGetTimeDeltaFromDate("483648-01-01", "1-01-01", "P483647Y");
	assertGetTimeDeltaFromDate("5880000-01-01", "1-01-01", "P5879999Y");
	assertGetTimeDeltaFromDate("5880000-12-01", "1-01-01", "P5879999Y11M");
	assertGetTimeDeltaFromDate("5880000-12-31", "1-01-01", "P5879999Y11M30D");

	assertGetTimeDeltaFromDate("5880001-01-01", "1-01-01", "P5880000Y");
	assertGetTimeDeltaFromDate("5880000-01-01", "0-01-01", "P5880000Y");
	assertGetTimeDeltaFromDate("7483648-01-01", "1-01-01", "P7483647Y");
	assertGetTimeDeltaFromDate("47483648-01-01", "1-01-01", "P47483647Y");
	assertGetTimeDeltaFromDate("147483648-01-01", "1-01-01", "P147483647Y");
	assertGetTimeDeltaFromDate("1147483648-01-01", "1-01-01", "P1147483647Y");
	assertGetTimeDeltaFromDate("2147483647-01-01", "1-01-01", "P2147483646Y");
	assertGetTimeDeltaFromDate("2147483647-01-01", "0-01-01", "P2147483647Y");
	assertGetTimeDeltaFromDate("1-01-01", "2147483647-01-01", "-P2147483646Y");
	assertGetTimeDeltaFromDate("0-01-01", "2147483647-01-01", "-P2147483647Y");
}
END_TEST

START_TEST(test_getTimeDeltaFromDate_YearOf365Days)
{
	assertGetTimeDeltaFromDate("-2147483648-01-01", "-2147483648-01-01", "PT00.000S");
	assertGetTimeDeltaFromDate("0-01-01", "0-01-01", "PT00.000S");
	assertGetTimeDeltaFromDate("2147483647-12-31", "2147483647-12-31", "PT00.000S");

	assertGetTimeDeltaFromDate("0-01-02", "0-01-01", "P01D");
	assertGetTimeDeltaFromDate("0-02-01", "0-01-01", "P01M");
	assertGetTimeDeltaFromDate("1-01-01", "0-01-01", "P1Y");
	assertGetTimeDeltaFromDate("2-01-01", "1-01-01", "P1Y");
	assertGetTimeDeltaFromDate("0-01-01", "0-01-02", "-P01D");
	assertGetTimeDeltaFromDate("0-01-01", "0-02-01", "-P01M");
	assertGetTimeDeltaFromDate("0-01-01", "1-01-01", "-P1Y");
	assertGetTimeDeltaFromDate("1-01-01", "2-01-01", "-P1Y");
	assertGetTimeDeltaFromDate("1-01-01", "2-12-01", "-P1Y11M");
	assertGetTimeDeltaFromDate("1-01-01", "2-12-31", "-P1Y11M30D");

	assertGetTimeDeltaFromDate("-1-12-31", "0-01-01", "-P01D");
	assertGetTimeDeltaFromDate("-1-12-01", "0-01-01", "-P01M");
	assertGetTimeDeltaFromDate("-1-01-01", "0-01-01", "-P1Y");
	assertGetTimeDeltaFromDate("-2-01-01", "-1-01-01", "-P1Y");
	assertGetTimeDeltaFromDate("0-01-01", "-1-12-31", "P01D");
	assertGetTimeDeltaFromDate("0-01-01", "-1-12-01", "P01M");
	assertGetTimeDeltaFromDate("0-01-01", "-1-01-01", "P1Y");
	assertGetTimeDeltaFromDate("-1-01-01", "-2-01-01", "P1Y");

	assertGetTimeDeltaFromDate("2012-03-12", "2012-03-12", "PT00.000S");
	assertGetTimeDeltaFromDate("2012-03-12", "2012-03-11", "P01D");
	assertGetTimeDeltaFromDate("2012-03-12", "2012-03-01", "P11D");
	assertGetTimeDeltaFromDate("2012-03-12", "2012-02-28", "P12D");
	assertGetTimeDeltaFromDate("2012-03-12", "2012-02-13", "P27D");
	assertGetTimeDeltaFromDate("2012-03-12", "2012-02-12", "P01M");
	assertGetTimeDeltaFromDate("2012-03-31", "2012-02-28", "P01M03D");
	assertGetTimeDeltaFromDate("2012-02-28", "2012-01-31", "P28D");
	assertGetTimeDeltaFromDate("2012-03-12", "2012-02-01", "P01M11D");
	assertGetTimeDeltaFromDate("2012-03-12", "2012-01-25", "P01M18D");
	assertGetTimeDeltaFromDate("2012-03-12", "2011-03-12", "P1Y");
	assertGetTimeDeltaFromDate("2012-03-12", "2011-03-11", "P1Y01D");
	assertGetTimeDeltaFromDate("2012-03-12", "2011-03-05", "P1Y07D");
	assertGetTimeDeltaFromDate("2012-03-12", "2011-03-01", "P1Y11D");
	assertGetTimeDeltaFromDate("2012-03-12", "2011-02-28", "P1Y12D");
	assertGetTimeDeltaFromDate("2012-03-12", "2011-02-13", "P1Y27D");
	assertGetTimeDeltaFromDate("2012-03-12", "2011-02-12", "P1Y01M");
	assertGetTimeDeltaFromDate("2012-03-12", "2011-02-07", "P1Y01M05D");
	assertGetTimeDeltaFromDate("2012-03-12", "2011-01-30", "P1Y01M13D");
	assertGetTimeDeltaFromDate("2012-03-12", "2010-09-25", "P1Y05M17D");

	assertGetTimeDeltaFromDate("2012-03-12", "2012-03-12", "PT00.000S");
	assertGetTimeDeltaFromDate("2012-03-11", "2012-03-12", "-P01D");
	assertGetTimeDeltaFromDate("2012-03-01", "2012-03-12", "-P11D");
	assertGetTimeDeltaFromDate("2012-02-28", "2012-03-12", "-P12D");
	assertGetTimeDeltaFromDate("2012-02-13", "2012-03-12", "-P27D");
	assertGetTimeDeltaFromDate("2012-02-12", "2012-03-12", "-P01M");
	assertGetTimeDeltaFromDate("2012-02-28", "2012-03-31", "-P01M03D");
	assertGetTimeDeltaFromDate("2012-01-31", "2012-02-28", "-P28D");
	assertGetTimeDeltaFromDate("2012-02-01", "2012-03-12", "-P01M11D");
	assertGetTimeDeltaFromDate("2012-01-25", "2012-03-12", "-P01M15D");
	assertGetTimeDeltaFromDate("2011-03-12", "2012-03-12", "-P1Y");
	assertGetTimeDeltaFromDate("2011-03-11", "2012-03-12", "-P1Y01D");
	assertGetTimeDeltaFromDate("2011-03-05", "2012-03-12", "-P1Y07D");
	assertGetTimeDeltaFromDate("2011-03-01", "2012-03-12", "-P1Y11D");
	assertGetTimeDeltaFromDate("2011-02-28", "2012-03-12", "-P1Y12D");
	assertGetTimeDeltaFromDate("2011-02-13", "2012-03-12", "-P1Y27D");
	assertGetTimeDeltaFromDate("2011-02-12", "2012-03-12", "-P1Y01M");
	assertGetTimeDeltaFromDate("2011-02-07", "2012-03-12", "-P1Y01M05D");
	assertGetTimeDeltaFromDate("2011-01-30", "2012-03-12", "-P1Y01M10D");
	assertGetTimeDeltaFromDate("2010-09-25", "2012-03-12", "-P1Y05M15D");

	assertGetTimeDeltaFromDate("-2012-03-12", "-2012-03-12", "PT00.000S");
	assertGetTimeDeltaFromDate("-2012-03-12", "-2012-03-11", "P01D");
	assertGetTimeDeltaFromDate("-2012-03-12", "-2012-03-01", "P11D");
	assertGetTimeDeltaFromDate("-2012-03-12", "-2012-02-28", "P12D");
	assertGetTimeDeltaFromDate("-2012-03-12", "-2012-02-13", "P27D");
	assertGetTimeDeltaFromDate("-2012-03-12", "-2012-02-12", "P01M");
	assertGetTimeDeltaFromDate("-2012-03-31", "-2012-02-28", "P01M03D");
	assertGetTimeDeltaFromDate("-2012-02-28", "-2012-01-31", "P28D");
	assertGetTimeDeltaFromDate("-2012-03-12", "-2012-02-01", "P01M11D");
	assertGetTimeDeltaFromDate("-2012-03-12", "-2012-01-25", "P01M18D");
	assertGetTimeDeltaFromDate("-2012-03-12", "-2013-03-12", "P1Y");
	assertGetTimeDeltaFromDate("-2012-03-12", "-2013-03-11", "P1Y01D");
	assertGetTimeDeltaFromDate("-2012-03-12", "-2013-03-05", "P1Y07D");
	assertGetTimeDeltaFromDate("-2012-03-12", "-2013-03-01", "P1Y11D");
	assertGetTimeDeltaFromDate("-2012-03-12", "-2013-02-28", "P1Y12D");
	assertGetTimeDeltaFromDate("-2012-03-12", "-2013-02-13", "P1Y27D");
	assertGetTimeDeltaFromDate("-2012-03-12", "-2013-02-12", "P1Y01M");
	assertGetTimeDeltaFromDate("-2012-03-12", "-2013-02-07", "P1Y01M05D");
	assertGetTimeDeltaFromDate("-2012-03-12", "-2013-01-30", "P1Y01M13D");
	assertGetTimeDeltaFromDate("-2012-03-12", "-2014-09-25", "P1Y05M17D");

	assertGetTimeDeltaFromDate("-2012-03-12", "-2012-03-12", "PT00.000S");
	assertGetTimeDeltaFromDate("-2012-03-11", "-2012-03-12", "-P01D");
	assertGetTimeDeltaFromDate("-2012-03-01", "-2012-03-12", "-P11D");
	assertGetTimeDeltaFromDate("-2012-03-01", "-2012-03-12", "-P11D");
	assertGetTimeDeltaFromDate("-2012-02-28", "-2012-03-12", "-P12D");
	assertGetTimeDeltaFromDate("-2012-02-13", "-2012-03-12", "-P27D");
	assertGetTimeDeltaFromDate("-2012-02-12", "-2012-03-12", "-P01M");
	assertGetTimeDeltaFromDate("-2012-02-28", "-2012-03-31", "-P01M03D");
	assertGetTimeDeltaFromDate("-2012-01-31", "-2012-02-28", "-P28D");
	assertGetTimeDeltaFromDate("-2012-02-01", "-2012-03-12", "-P01M11D");
	assertGetTimeDeltaFromDate("-2012-01-25", "-2012-03-12", "-P01M15D");
	assertGetTimeDeltaFromDate("-2013-03-12", "-2012-03-12", "-P1Y");
	assertGetTimeDeltaFromDate("-2013-03-11", "-2012-03-12", "-P1Y01D");
	assertGetTimeDeltaFromDate("-2013-03-05", "-2012-03-12", "-P1Y07D");
	assertGetTimeDeltaFromDate("-2013-03-01", "-2012-03-12", "-P1Y11D");
	assertGetTimeDeltaFromDate("-2013-02-28", "-2012-03-12", "-P1Y12D");
	assertGetTimeDeltaFromDate("-2013-02-13", "-2012-03-12", "-P1Y27D");
	assertGetTimeDeltaFromDate("-2013-02-12", "-2012-03-12", "-P1Y01M");
	assertGetTimeDeltaFromDate("-2013-02-07", "-2012-03-12", "-P1Y01M05D");
	assertGetTimeDeltaFromDate("-2013-01-30", "-2012-03-12", "-P1Y01M10D");
	assertGetTimeDeltaFromDate("-2014-09-25", "-2012-03-12", "-P1Y05M15D");
}
END_TEST

START_TEST(test_getTimeDeltaFromDate_YearOf365Days_HighDeltas)
{
	assertGetTimeDeltaFromDate("-2147483648-01-01", "-2147483648-01-01", "PT00.000S");
	assertGetTimeDeltaFromDate("-2147483648-01-01", "-2147483647-01-01", "-P1Y");
	assertGetTimeDeltaFromDate("-2147483647-01-01", "-2147483648-01-01", "P1Y");
	assertGetTimeDeltaFromDate("2147483647-01-01", "2147483647-01-01", "PT00.000S");
	assertGetTimeDeltaFromDate("2147483647-01-01", "2147483646-01-01", "P1Y");
	assertGetTimeDeltaFromDate("2147483646-01-01", "2147483647-01-01", "-P1Y");

	assertGetTimeDeltaFromDate("-83648-01-01", "-1-01-01", "-P83647Y");
	assertGetTimeDeltaFromDate("-483648-01-01", "-1-01-01", "-P483647Y");
	assertGetTimeDeltaFromDate("-5880000-01-01", "-1-01-01", "-P5879999Y");
	assertGetTimeDeltaFromDate("-5880001-02-01", "-1-01-01", "-P5879999Y11M");
	assertGetTimeDeltaFromDate("-5880001-01-02", "-1-01-01", "-P5879999Y11M30D");

	assertGetTimeDeltaFromDate("-5880001-01-01", "-1-01-01", "-P5880000Y");
	assertGetTimeDeltaFromDate("-5880000-01-01", "0-01-01", "-P5880000Y");
	assertGetTimeDeltaFromDate("-7483648-01-01", "-1-01-01", "-P7483647Y");
	assertGetTimeDeltaFromDate("-47483648-01-01", "-1-01-01", "-P47483647Y");
	assertGetTimeDeltaFromDate("-147483648-01-01", "-1-01-01", "-P147483647Y");
	assertGetTimeDeltaFromDate("-1147483648-01-01", "-1-01-01", "-P1147483647Y");
	assertGetTimeDeltaFromDate("-2147483648-01-01", "-1-01-01", "-P2147483647Y");
	assertGetTimeDeltaFromDate("-2147483648-01-01", "0-01-01", "-P2147483648Y");
	assertGetTimeDeltaFromDate("-1-01-01", "-2147483648-01-01", "P2147483647Y");
	assertGetTimeDeltaFromDate("0-01-01", "-2147483647-01-01", "P2147483647Y");

	assertGetTimeDeltaFromDate("83648-01-01", "1-01-01", "P83647Y");
	assertGetTimeDeltaFromDate("483648-01-01", "1-01-01", "P483647Y");
	assertGetTimeDeltaFromDate("5880000-01-01", "1-01-01", "P5879999Y");
	assertGetTimeDeltaFromDate("5880000-12-01", "1-01-01", "P5879999Y11M");
	assertGetTimeDeltaFromDate("5880000-12-31", "1-01-01", "P5879999Y11M30D");

	assertGetTimeDeltaFromDate("5880001-01-01", "1-01-01", "P5880000Y");
	assertGetTimeDeltaFromDate("5880000-01-01", "0-01-01", "P5880000Y");
	assertGetTimeDeltaFromDate("7483648-01-01", "1-01-01", "P7483647Y");
	assertGetTimeDeltaFromDate("47483648-01-01", "1-01-01", "P47483647Y");
	assertGetTimeDeltaFromDate("147483648-01-01", "1-01-01", "P147483647Y");
	assertGetTimeDeltaFromDate("1147483648-01-01", "1-01-01", "P1147483647Y");
	assertGetTimeDeltaFromDate("2147483647-01-01", "1-01-01", "P2147483646Y");
	assertGetTimeDeltaFromDate("2147483647-01-01", "0-01-01", "P2147483647Y");
	assertGetTimeDeltaFromDate("1-01-01", "2147483647-01-01", "-P2147483646Y");
	assertGetTimeDeltaFromDate("0-01-01", "2147483647-01-01", "-P2147483647Y");
}
END_TEST

START_TEST(test_getTimeDeltaFromDate_YearOf360Days)
{
	assertGetTimeDeltaFromDate("-2147483648-01-01", "-2147483648-01-01", "PT00.000S");
	assertGetTimeDeltaFromDate("0-01-01", "0-01-01", "PT00.000S");
	assertGetTimeDeltaFromDate("2147483647-12-31", "2147483647-12-31", "PT00.000S");

	assertGetTimeDeltaFromDate("0-01-02", "0-01-01", "P01D");
	assertGetTimeDeltaFromDate("0-02-01", "0-01-01", "P01M");
	assertGetTimeDeltaFromDate("1-01-01", "0-01-01", "P1Y");
	assertGetTimeDeltaFromDate("2-01-01", "1-01-01", "P1Y");
	assertGetTimeDeltaFromDate("0-01-01", "0-01-02", "-P01D");
	assertGetTimeDeltaFromDate("0-01-01", "0-02-01", "-P01M");
	assertGetTimeDeltaFromDate("0-01-01", "1-01-01", "-P1Y");
	assertGetTimeDeltaFromDate("1-01-01", "2-01-01", "-P1Y");
	assertGetTimeDeltaFromDate("1-01-01", "2-12-01", "-P1Y11M");
	assertGetTimeDeltaFromDate("1-01-01", "2-12-30", "-P1Y11M29D");

	assertGetTimeDeltaFromDate("-1-12-30", "0-01-01", "-P01D");
	assertGetTimeDeltaFromDate("-1-12-01", "0-01-01", "-P01M");
	assertGetTimeDeltaFromDate("-1-01-01", "0-01-01", "-P1Y");
	assertGetTimeDeltaFromDate("-2-01-01", "-1-01-01", "-P1Y");
	assertGetTimeDeltaFromDate("0-01-01", "-1-12-30", "P01D");
	assertGetTimeDeltaFromDate("0-01-01", "-1-12-01", "P01M");
	assertGetTimeDeltaFromDate("0-01-01", "-1-01-01", "P1Y");
	assertGetTimeDeltaFromDate("-1-01-01", "-2-01-01", "P1Y");

	assertGetTimeDeltaFromDate("2012-03-12", "2012-03-12", "PT00.000S");
	assertGetTimeDeltaFromDate("2012-03-12", "2012-03-11", "P01D");
	assertGetTimeDeltaFromDate("2012-03-12", "2012-03-01", "P11D");
	assertGetTimeDeltaFromDate("2012-03-12", "2012-02-28", "P14D");
	assertGetTimeDeltaFromDate("2012-03-12", "2012-02-13", "P29D");
	assertGetTimeDeltaFromDate("2012-03-12", "2012-02-12", "P01M");
	assertGetTimeDeltaFromDate("2012-03-30", "2012-02-28", "P01M02D");
	assertGetTimeDeltaFromDate("2012-02-28", "2012-01-30", "P28D");
	assertGetTimeDeltaFromDate("2012-03-12", "2012-02-01", "P01M11D");
	assertGetTimeDeltaFromDate("2012-03-12", "2012-01-25", "P01M17D");
	assertGetTimeDeltaFromDate("2012-03-12", "2011-03-12", "P1Y");
	assertGetTimeDeltaFromDate("2012-03-12", "2011-03-11", "P1Y01D");
	assertGetTimeDeltaFromDate("2012-03-12", "2011-03-05", "P1Y07D");
	assertGetTimeDeltaFromDate("2012-03-12", "2011-03-01", "P1Y11D");
	assertGetTimeDeltaFromDate("2012-03-12", "2011-02-28", "P1Y14D");
	assertGetTimeDeltaFromDate("2012-03-12", "2011-02-13", "P1Y29D");
	assertGetTimeDeltaFromDate("2012-03-12", "2011-02-12", "P1Y01M");
	assertGetTimeDeltaFromDate("2012-03-12", "2011-02-07", "P1Y01M05D");
	assertGetTimeDeltaFromDate("2012-03-12", "2011-01-30", "P1Y01M12D");
	assertGetTimeDeltaFromDate("2012-03-12", "2010-09-25", "P1Y05M17D");

	assertGetTimeDeltaFromDate("2012-03-12", "2012-03-12", "PT00.000S");
	assertGetTimeDeltaFromDate("2012-03-11", "2012-03-12", "-P01D");
	assertGetTimeDeltaFromDate("2012-03-01", "2012-03-12", "-P11D");
	assertGetTimeDeltaFromDate("2012-03-01", "2012-03-12", "-P11D");
	assertGetTimeDeltaFromDate("2012-02-28", "2012-03-12", "-P14D");
	assertGetTimeDeltaFromDate("2012-02-13", "2012-03-12", "-P29D");
	assertGetTimeDeltaFromDate("2012-02-12", "2012-03-12", "-P01M");
	assertGetTimeDeltaFromDate("2012-02-28", "2012-03-30", "-P01M02D");
	assertGetTimeDeltaFromDate("2012-01-30", "2012-02-28", "-P28D");
	assertGetTimeDeltaFromDate("2012-02-01", "2012-03-12", "-P01M11D");
	assertGetTimeDeltaFromDate("2012-01-25", "2012-03-12", "-P01M17D");
	assertGetTimeDeltaFromDate("2011-03-12", "2012-03-12", "-P1Y");
	assertGetTimeDeltaFromDate("2011-03-11", "2012-03-12", "-P1Y01D");
	assertGetTimeDeltaFromDate("2011-03-05", "2012-03-12", "-P1Y07D");
	assertGetTimeDeltaFromDate("2011-03-01", "2012-03-12", "-P1Y11D");
	assertGetTimeDeltaFromDate("2011-02-28", "2012-03-12", "-P1Y14D");
	assertGetTimeDeltaFromDate("2011-02-13", "2012-03-12", "-P1Y29D");
	assertGetTimeDeltaFromDate("2011-02-12", "2012-03-12", "-P1Y01M");
	assertGetTimeDeltaFromDate("2011-02-07", "2012-03-12", "-P1Y01M05D");
	assertGetTimeDeltaFromDate("2011-01-30", "2012-03-12", "-P1Y01M12D");
	assertGetTimeDeltaFromDate("2010-09-25", "2012-03-12", "-P1Y05M17D");

	assertGetTimeDeltaFromDate("-2012-03-12", "-2012-03-12", "PT00.000S");
	assertGetTimeDeltaFromDate("-2012-03-12", "-2012-03-11", "P01D");
	assertGetTimeDeltaFromDate("-2012-03-12", "-2012-03-01", "P11D");
	assertGetTimeDeltaFromDate("-2012-03-12", "-2012-03-01", "P11D");
	assertGetTimeDeltaFromDate("-2012-03-12", "-2012-02-28", "P14D");
	assertGetTimeDeltaFromDate("-2012-03-12", "-2012-02-13", "P29D");
	assertGetTimeDeltaFromDate("-2012-03-12", "-2012-02-12", "P01M");
	assertGetTimeDeltaFromDate("-2012-03-30", "-2012-02-28", "P01M02D");
	assertGetTimeDeltaFromDate("-2012-02-28", "-2012-01-30", "P28D");
	assertGetTimeDeltaFromDate("-2012-03-12", "-2012-02-01", "P01M11D");
	assertGetTimeDeltaFromDate("-2012-03-12", "-2012-01-25", "P01M17D");
	assertGetTimeDeltaFromDate("-2012-03-12", "-2013-03-12", "P1Y");
	assertGetTimeDeltaFromDate("-2012-03-12", "-2013-03-11", "P1Y01D");
	assertGetTimeDeltaFromDate("-2012-03-12", "-2013-03-05", "P1Y07D");
	assertGetTimeDeltaFromDate("-2012-03-12", "-2013-03-01", "P1Y11D");
	assertGetTimeDeltaFromDate("-2012-03-12", "-2013-02-28", "P1Y14D");
	assertGetTimeDeltaFromDate("-2012-03-12", "-2013-02-13", "P1Y29D");
	assertGetTimeDeltaFromDate("-2012-03-12", "-2013-02-12", "P1Y01M");
	assertGetTimeDeltaFromDate("-2012-03-12", "-2013-02-07", "P1Y01M05D");
	assertGetTimeDeltaFromDate("-2012-03-12", "-2013-01-30", "P1Y01M12D");
	assertGetTimeDeltaFromDate("-2012-03-12", "-2014-09-25", "P1Y05M17D");

	assertGetTimeDeltaFromDate("-2012-03-12", "-2012-03-12", "PT00.000S");
	assertGetTimeDeltaFromDate("-2012-03-11", "-2012-03-12", "-P01D");
	assertGetTimeDeltaFromDate("-2012-03-01", "-2012-03-12", "-P11D");
	assertGetTimeDeltaFromDate("-2012-02-28", "-2012-03-12", "-P14D");
	assertGetTimeDeltaFromDate("-2012-02-13", "-2012-03-12", "-P29D");
	assertGetTimeDeltaFromDate("-2012-02-12", "-2012-03-12", "-P01M");
	assertGetTimeDeltaFromDate("-2012-02-28", "-2012-03-30", "-P01M02D");
	assertGetTimeDeltaFromDate("-2012-01-30", "-2012-02-28", "-P28D");
	assertGetTimeDeltaFromDate("-2012-02-01", "-2012-03-12", "-P01M11D");
	assertGetTimeDeltaFromDate("-2012-01-25", "-2012-03-12", "-P01M17D");
	assertGetTimeDeltaFromDate("-2013-03-12", "-2012-03-12", "-P1Y");
	assertGetTimeDeltaFromDate("-2013-03-11", "-2012-03-12", "-P1Y01D");
	assertGetTimeDeltaFromDate("-2013-03-05", "-2012-03-12", "-P1Y07D");
	assertGetTimeDeltaFromDate("-2013-03-01", "-2012-03-12", "-P1Y11D");
	assertGetTimeDeltaFromDate("-2013-02-28", "-2012-03-12", "-P1Y14D");
	assertGetTimeDeltaFromDate("-2013-02-13", "-2012-03-12", "-P1Y29D");
	assertGetTimeDeltaFromDate("-2013-02-12", "-2012-03-12", "-P1Y01M");
	assertGetTimeDeltaFromDate("-2013-02-07", "-2012-03-12", "-P1Y01M05D");
	assertGetTimeDeltaFromDate("-2013-01-30", "-2012-03-12", "-P1Y01M12D");
	assertGetTimeDeltaFromDate("-2014-09-25", "-2012-03-12", "-P1Y05M17D");
}
END_TEST

START_TEST(test_getTimeDeltaFromDate_YearOf360Days_HighDeltas)
{
	assertGetTimeDeltaFromDate("-2147483648-01-01", "-2147483648-01-01", "PT00.000S");
	assertGetTimeDeltaFromDate("-2147483648-01-01", "-2147483647-01-01", "-P1Y");
	assertGetTimeDeltaFromDate("-2147483647-01-01", "-2147483648-01-01", "P1Y");
	assertGetTimeDeltaFromDate("2147483647-01-01", "2147483647-01-01", "PT00.000S");
	assertGetTimeDeltaFromDate("2147483647-01-01", "2147483646-01-01", "P1Y");
	assertGetTimeDeltaFromDate("2147483646-01-01", "2147483647-01-01", "-P1Y");

	assertGetTimeDeltaFromDate("-83648-01-01", "-1-01-01", "-P83647Y");
	assertGetTimeDeltaFromDate("-483648-01-01", "-1-01-01", "-P483647Y");
	assertGetTimeDeltaFromDate("-5880000-01-01", "-1-01-01", "-P5879999Y");
	assertGetTimeDeltaFromDate("-5880001-02-01", "-1-01-01", "-P5879999Y11M");
	assertGetTimeDeltaFromDate("-5880001-01-02", "-1-01-01", "-P5879999Y11M29D");

	assertGetTimeDeltaFromDate("-5880001-01-01", "-1-01-01", "-P5880000Y");
	assertGetTimeDeltaFromDate("-5880000-01-01", "0-01-01", "-P5880000Y");
	assertGetTimeDeltaFromDate("-7483648-01-01", "-1-01-01", "-P7483647Y");
	assertGetTimeDeltaFromDate("-47483648-01-01", "-1-01-01", "-P47483647Y");
	assertGetTimeDeltaFromDate("-147483648-01-01", "-1-01-01", "-P147483647Y");
	assertGetTimeDeltaFromDate("-1147483648-01-01", "-1-01-01", "-P1147483647Y");
	assertGetTimeDeltaFromDate("-2147483648-01-01", "-1-01-01", "-P2147483647Y");
	assertGetTimeDeltaFromDate("-2147483648-01-01", "0-01-01", "-P2147483648Y");
	assertGetTimeDeltaFromDate("-1-01-01", "-2147483648-01-01", "P2147483647Y");
	assertGetTimeDeltaFromDate("0-01-01", "-2147483647-01-01", "P2147483647Y");

	assertGetTimeDeltaFromDate("83648-01-01", "1-01-01", "P83647Y");
	assertGetTimeDeltaFromDate("483648-01-01", "1-01-01", "P483647Y");
	assertGetTimeDeltaFromDate("5880000-01-01", "1-01-01", "P5879999Y");
	assertGetTimeDeltaFromDate("5880000-12-01", "1-01-01", "P5879999Y11M");
	assertGetTimeDeltaFromDate("5880000-12-30", "1-01-01", "P5879999Y11M29D");

	assertGetTimeDeltaFromDate("5880001-01-01", "1-01-01", "P5880000Y");
	assertGetTimeDeltaFromDate("5880000-01-01", "0-01-01", "P5880000Y");
	assertGetTimeDeltaFromDate("7483648-01-01", "1-01-01", "P7483647Y");
	assertGetTimeDeltaFromDate("47483648-01-01", "1-01-01", "P47483647Y");
	assertGetTimeDeltaFromDate("147483648-01-01", "1-01-01", "P147483647Y");
	assertGetTimeDeltaFromDate("1147483648-01-01", "1-01-01", "P1147483647Y");
	assertGetTimeDeltaFromDate("2147483647-01-01", "1-01-01", "P2147483646Y");
	assertGetTimeDeltaFromDate("2147483647-01-01", "0-01-01", "P2147483647Y");
	assertGetTimeDeltaFromDate("1-01-01", "2147483647-01-01", "-P2147483646Y");
	assertGetTimeDeltaFromDate("0-01-01", "2147483647-01-01", "-P2147483647Y");
}
END_TEST


START_TEST(test_getTimeDeltaFromDateTime_ProlepticGregorian)
{
	assertGetTimeDeltaFromDateTime("-2147483648-01-01T00:00:00.000", "-2147483648-01-01T00:00:00.000", "PT00.000S");
	assertGetTimeDeltaFromDateTime("0-01-01T00:00:00.000", "0-01-01T00:00:00.000", "PT00.000S");
	assertGetTimeDeltaFromDateTime("2147483647-12-31T23:59:59.999", "2147483647-12-31T23:59:59.999", "PT00.000S");

	assertGetTimeDeltaFromDateTime("0-01-01T00:00:00.001", "0-01-01T00:00:00.000", "PT00.001S");
	assertGetTimeDeltaFromDateTime("0-01-01T00:00:01.000", "0-01-01T00:00:00.000", "PT01.000S");
	assertGetTimeDeltaFromDateTime("0-01-01T00:01:00.000", "0-01-01T00:00:00.000", "PT01M");
	assertGetTimeDeltaFromDateTime("0-01-01T01:00:00.000", "0-01-01T00:00:00.000", "PT01H");
	assertGetTimeDeltaFromDateTime("0-01-02T00:00:00.000", "0-01-01T00:00:00.000", "P01D");
	assertGetTimeDeltaFromDateTime("0-02-01T00:00:00.000", "0-01-01T00:00:00.000", "P01M");
	assertGetTimeDeltaFromDateTime("1-01-01T00:00:00.000", "0-01-01T00:00:00.000", "P1Y");
	assertGetTimeDeltaFromDateTime("2-01-01T00:00:00.000", "1-01-01T00:00:00.000", "P1Y");
	assertGetTimeDeltaFromDateTime("0-01-01T00:00:00.000", "0-01-01T00:00:00.001", "-PT00.001S");
	assertGetTimeDeltaFromDateTime("0-01-01T00:00:00.000", "0-01-01T00:00:01.000", "-PT01.000S");
	assertGetTimeDeltaFromDateTime("0-01-01T00:00:00.000", "0-01-01T00:01:00.000", "-PT01M");
	assertGetTimeDeltaFromDateTime("0-01-01T00:00:00.000", "0-01-01T01:00:00.000", "-PT01H");
	assertGetTimeDeltaFromDateTime("0-01-01T00:00:00.000", "0-01-02T00:00:00.000", "-P01D");
	assertGetTimeDeltaFromDateTime("0-01-01T00:00:00.000", "0-02-01T00:00:00.000", "-P01M");
	assertGetTimeDeltaFromDateTime("0-01-01T00:00:00.000", "1-01-01T00:00:00.000", "-P1Y");
	assertGetTimeDeltaFromDateTime("1-01-01T00:00:00.000", "2-01-01T00:00:00.000", "-P1Y");
	assertGetTimeDeltaFromDateTime("1-01-01T00:00:00.000", "2-12-01T00:00:00.000", "-P1Y11M");
	assertGetTimeDeltaFromDateTime("1-01-01T00:00:00.000", "2-12-31T00:00:00.000", "-P1Y11M30D");
	assertGetTimeDeltaFromDateTime("1-01-01T00:00:00.000", "2-12-31T01:00:00.000", "-P1Y11M30DT01H");
	assertGetTimeDeltaFromDateTime("1-01-01T00:00:00.000", "2-12-31T01:01:00.000", "-P1Y11M30DT01H01M");
	assertGetTimeDeltaFromDateTime("1-01-01T00:00:00.000", "2-12-31T01:01:01.000", "-P1Y11M30DT01H01M01.000S");
	assertGetTimeDeltaFromDateTime("1-01-01T00:00:00.000", "2-12-31T01:01:01.001", "-P1Y11M30DT01H01M01.001S");

	assertGetTimeDeltaFromDateTime("2014-09-04T16:23:45.123", "2014-09-04T16:23:45.123", "PT00.000S");
	assertGetTimeDeltaFromDateTime("2014-09-04T16:23:45.123", "2014-09-04T16:23:44.456", "PT00.667S");
	assertGetTimeDeltaFromDateTime("2014-09-04T16:23:45.123", "2014-09-04T16:23:46.456", "-PT01.333S");
	assertGetTimeDeltaFromDateTime("2014-09-04T16:23:45.123", "2012-01-16T10:15:46.456", "P2Y07M19DT06H07M58.667S");
}
END_TEST

START_TEST(test_getTimeDeltaFromDateTime_ProlepticGregorian_HighDeltas)
{
	assertGetTimeDeltaFromDateTime("5880000-01-01T00:00:00.000", "1-01-01T00:00:00.000", "P5879999Y");
	assertGetTimeDeltaFromDateTime("5880000-12-01T00:00:00.000", "1-01-01T00:00:00.000", "P5879999Y11M");
	assertGetTimeDeltaFromDateTime("5880000-12-31T00:00:00.000", "1-01-01T00:00:00.000", "P5879999Y11M30D");
	assertGetTimeDeltaFromDateTime("5880000-12-31T23:59:59.999", "1-01-01T00:00:00.000", "P5879999Y11M30DT23H59M59.999S");

	assertGetTimeDeltaFromDateTime("5880001-01-01T00:00:00.000", "1-01-01T00:00:00.000", "P5880000Y");
}
END_TEST

START_TEST(test_getTimeDeltaFromDateTime_YearOf365Days)
{
	assertGetTimeDeltaFromDateTime("-2147483648-01-01T00:00:00.000", "-2147483648-01-01T00:00:00.000", "PT00.000S");
	assertGetTimeDeltaFromDateTime("0-01-01T00:00:00.000", "0-01-01T00:00:00.000", "PT00.000S");
	assertGetTimeDeltaFromDateTime("2147483647-12-31T23:59:59.999", "2147483647-12-31T23:59:59.999", "PT00.000S");

	assertGetTimeDeltaFromDateTime("0-01-01T00:00:00.001", "0-01-01T00:00:00.000", "PT00.001S");
	assertGetTimeDeltaFromDateTime("0-01-01T00:00:01.000", "0-01-01T00:00:00.000", "PT01.000S");
	assertGetTimeDeltaFromDateTime("0-01-01T00:01:00.000", "0-01-01T00:00:00.000", "PT01M");
	assertGetTimeDeltaFromDateTime("0-01-01T01:00:00.000", "0-01-01T00:00:00.000", "PT01H");
	assertGetTimeDeltaFromDateTime("0-01-02T00:00:00.000", "0-01-01T00:00:00.000", "P01D");
	assertGetTimeDeltaFromDateTime("0-02-01T00:00:00.000", "0-01-01T00:00:00.000", "P01M");
	assertGetTimeDeltaFromDateTime("1-01-01T00:00:00.000", "0-01-01T00:00:00.000", "P1Y");
	assertGetTimeDeltaFromDateTime("2-01-01T00:00:00.000", "1-01-01T00:00:00.000", "P1Y");
	assertGetTimeDeltaFromDateTime("0-01-01T00:00:00.000", "0-01-01T00:00:00.001", "-PT00.001S");
	assertGetTimeDeltaFromDateTime("0-01-01T00:00:00.000", "0-01-01T00:00:01.000", "-PT01.000S");
	assertGetTimeDeltaFromDateTime("0-01-01T00:00:00.000", "0-01-01T00:01:00.000", "-PT01M");
	assertGetTimeDeltaFromDateTime("0-01-01T00:00:00.000", "0-01-01T01:00:00.000", "-PT01H");
	assertGetTimeDeltaFromDateTime("0-01-01T00:00:00.000", "0-01-02T00:00:00.000", "-P01D");
	assertGetTimeDeltaFromDateTime("0-01-01T00:00:00.000", "0-02-01T00:00:00.000", "-P01M");
	assertGetTimeDeltaFromDateTime("0-01-01T00:00:00.000", "1-01-01T00:00:00.000", "-P1Y");
	assertGetTimeDeltaFromDateTime("1-01-01T00:00:00.000", "2-01-01T00:00:00.000", "-P1Y");
	assertGetTimeDeltaFromDateTime("1-01-01T00:00:00.000", "2-12-01T00:00:00.000", "-P1Y11M");
	assertGetTimeDeltaFromDateTime("1-01-01T00:00:00.000", "2-12-31T00:00:00.000", "-P1Y11M30D");
	assertGetTimeDeltaFromDateTime("1-01-01T00:00:00.000", "2-12-31T01:00:00.000", "-P1Y11M30DT01H");
	assertGetTimeDeltaFromDateTime("1-01-01T00:00:00.000", "2-12-31T01:01:00.000", "-P1Y11M30DT01H01M");
	assertGetTimeDeltaFromDateTime("1-01-01T00:00:00.000", "2-12-31T01:01:01.000", "-P1Y11M30DT01H01M01.000S");
	assertGetTimeDeltaFromDateTime("1-01-01T00:00:00.000", "2-12-31T01:01:01.001", "-P1Y11M30DT01H01M01.001S");

	assertGetTimeDeltaFromDateTime("2014-09-04T16:23:45.123", "2014-09-04T16:23:45.123", "PT00.000S");
	assertGetTimeDeltaFromDateTime("2014-09-04T16:23:45.123", "2014-09-04T16:23:44.456", "PT00.667S");
	assertGetTimeDeltaFromDateTime("2014-09-04T16:23:45.123", "2014-09-04T16:23:46.456", "-PT01.333S");
	assertGetTimeDeltaFromDateTime("2014-09-04T16:23:45.123", "2012-01-16T10:15:46.456", "P2Y07M19DT06H07M58.667S");
}
END_TEST

START_TEST(test_getTimeDeltaFromDateTime_YearOf360Days)
{
	assertGetTimeDeltaFromDateTime("-2147483648-01-01T00:00:00.000", "-2147483648-01-01T00:00:00.000", "PT00.000S");
	assertGetTimeDeltaFromDateTime("0-01-01T00:00:00.000", "0-01-01T00:00:00.000", "PT00.000S");
	assertGetTimeDeltaFromDateTime("2147483647-12-31T23:59:59.999", "2147483647-12-31T23:59:59.999", "PT00.000S");

	assertGetTimeDeltaFromDateTime("0-01-01T00:00:00.001", "0-01-01T00:00:00.000", "PT00.001S");
	assertGetTimeDeltaFromDateTime("0-01-01T00:00:01.000", "0-01-01T00:00:00.000", "PT01.000S");
	assertGetTimeDeltaFromDateTime("0-01-01T00:01:00.000", "0-01-01T00:00:00.000", "PT01M");
	assertGetTimeDeltaFromDateTime("0-01-01T01:00:00.000", "0-01-01T00:00:00.000", "PT01H");
	assertGetTimeDeltaFromDateTime("0-01-02T00:00:00.000", "0-01-01T00:00:00.000", "P01D");
	assertGetTimeDeltaFromDateTime("0-02-01T00:00:00.000", "0-01-01T00:00:00.000", "P01M");
	assertGetTimeDeltaFromDateTime("1-01-01T00:00:00.000", "0-01-01T00:00:00.000", "P1Y");
	assertGetTimeDeltaFromDateTime("2-01-01T00:00:00.000", "1-01-01T00:00:00.000", "P1Y");
	assertGetTimeDeltaFromDateTime("0-01-01T00:00:00.000", "0-01-01T00:00:00.001", "-PT00.001S");
	assertGetTimeDeltaFromDateTime("0-01-01T00:00:00.000", "0-01-01T00:00:01.000", "-PT01.000S");
	assertGetTimeDeltaFromDateTime("0-01-01T00:00:00.000", "0-01-01T00:01:00.000", "-PT01M");
	assertGetTimeDeltaFromDateTime("0-01-01T00:00:00.000", "0-01-01T01:00:00.000", "-PT01H");
	assertGetTimeDeltaFromDateTime("0-01-01T00:00:00.000", "0-01-02T00:00:00.000", "-P01D");
	assertGetTimeDeltaFromDateTime("0-01-01T00:00:00.000", "0-02-01T00:00:00.000", "-P01M");
	assertGetTimeDeltaFromDateTime("0-01-01T00:00:00.000", "1-01-01T00:00:00.000", "-P1Y");
	assertGetTimeDeltaFromDateTime("1-01-01T00:00:00.000", "2-01-01T00:00:00.000", "-P1Y");
	assertGetTimeDeltaFromDateTime("1-01-01T00:00:00.000", "2-12-01T00:00:00.000", "-P1Y11M");
	assertGetTimeDeltaFromDateTime("1-01-01T00:00:00.000", "2-12-30T00:00:00.000", "-P1Y11M29D");
	assertGetTimeDeltaFromDateTime("1-01-01T00:00:00.000", "2-12-30T01:00:00.000", "-P1Y11M29DT01H");
	assertGetTimeDeltaFromDateTime("1-01-01T00:00:00.000", "2-12-30T01:01:00.000", "-P1Y11M29DT01H01M");
	assertGetTimeDeltaFromDateTime("1-01-01T00:00:00.000", "2-12-30T01:01:01.000", "-P1Y11M29DT01H01M01.000S");
	assertGetTimeDeltaFromDateTime("1-01-01T00:00:00.000", "2-12-30T01:01:01.001", "-P1Y11M29DT01H01M01.001S");

	assertGetTimeDeltaFromDateTime("2014-09-04T16:23:45.123", "2014-09-04T16:23:45.123", "PT00.000S");
	assertGetTimeDeltaFromDateTime("2014-09-04T16:23:45.123", "2014-09-04T16:23:44.456", "PT00.667S");
	assertGetTimeDeltaFromDateTime("2014-09-04T16:23:45.123", "2014-09-04T16:23:46.456", "-PT01.333S");
	assertGetTimeDeltaFromDateTime("2014-09-04T16:23:45.123", "2012-01-16T10:15:46.456", "P2Y07M18DT06H07M58.667S");
}
END_TEST

START_TEST(test_getTotalMilliSecondsTimeDelta_and_getTotalSecondsTimeDelta)
{
	assertTotalMilliSecondsAndTotalSeconds("PT00.000S", "2013-01-01T00:00:00.000", 0, 0);
	assertTotalMilliSecondsAndTotalSeconds("PT00.001S", "2013-01-01T00:00:00.000", 1, 0);
	assertTotalMilliSecondsAndTotalSeconds("PT01.000S", "2013-01-01T00:00:00.000", 1000, 1);
	assertTotalMilliSecondsAndTotalSeconds("PT01M", "2013-01-01T00:00:00.000", 60000, 60);
	assertTotalMilliSecondsAndTotalSeconds("PT01H", "2013-01-01T00:00:00.000", 3600000, 3600);
	assertTotalMilliSecondsAndTotalSeconds("P01D", "2013-01-01T00:00:00.000", 86400000, 86400);
	assertTotalMilliSecondsAndTotalSeconds("P01M", "2013-01-01T00:00:00.000", 2678400000, 2678400);
	assertTotalMilliSecondsAndTotalSeconds("P01M", "2013-02-01T00:00:00.000", 2419200000, 2419200);
	assertTotalMilliSecondsAndTotalSeconds("P01M", "2012-02-01T00:00:00.000", 2505600000, 2505600);
	assertTotalMilliSecondsAndTotalSeconds("P01M", "2013-03-01T00:00:00.000", 2678400000, 2678400);
	assertTotalMilliSecondsAndTotalSeconds("P01M", "2013-04-01T00:00:00.000", 2592000000, 2592000);
	assertTotalMilliSecondsAndTotalSeconds("P01M", "2013-05-01T00:00:00.000", 2678400000, 2678400);
	assertTotalMilliSecondsAndTotalSeconds("P01Y", "2013-01-01T00:00:00.000", 31536000000, 31536000);
	assertTotalMilliSecondsAndTotalSeconds("P01Y", "2012-01-01T00:00:00.000", 31622400000, 31622400);

	assertTotalMilliSecondsAndTotalSeconds("PT90.000S", "2013-01-01T00:00:00.000", 90000, 90);
	assertTotalMilliSecondsAndTotalSeconds("PT90M", "2013-01-01T00:00:00.000", 5400000, 5400);
	assertTotalMilliSecondsAndTotalSeconds("PT90H", "2013-01-01T00:00:00.000", 324000000, 324000);
	assertTotalMilliSecondsAndTotalSeconds("P90D", "2013-01-01T00:00:00.000", 7776000000, 7776000);

	assertTotalMilliSecondsAndTotalSeconds("-PT00.000S", "2013-01-01T00:00:00.000", 0, 0);
	assertTotalMilliSecondsAndTotalSeconds("-PT00.001S", "2013-01-01T00:00:00.000", -1, 0);
	assertTotalMilliSecondsAndTotalSeconds("-PT01.000S", "2013-01-01T00:00:00.000", -1000, -1);
	assertTotalMilliSecondsAndTotalSeconds("-PT01M", "2013-01-01T00:00:00.000", -60000, -60);
	assertTotalMilliSecondsAndTotalSeconds("-PT01H", "2013-01-01T00:00:00.000", -3600000, -3600);
	assertTotalMilliSecondsAndTotalSeconds("-P01M", "2013-01-01T00:00:00.000", -2678400000, -2678400);
	assertTotalMilliSecondsAndTotalSeconds("-P01M", "2013-02-01T00:00:00.000", -2678400000, -2678400);
	assertTotalMilliSecondsAndTotalSeconds("-P01M", "2013-03-01T00:00:00.000", -2419200000, -2419200);
	assertTotalMilliSecondsAndTotalSeconds("-P01M", "2012-03-01T00:00:00.000", -2505600000, -2505600);
	assertTotalMilliSecondsAndTotalSeconds("-P01M", "2013-04-01T00:00:00.000", -2678400000, -2678400);
	assertTotalMilliSecondsAndTotalSeconds("-P01M", "2013-05-01T00:00:00.000", -2592000000, -2592000);
	assertTotalMilliSecondsAndTotalSeconds("-P01Y", "2013-01-01T00:00:00.000", -31622400000, -31622400);
	assertTotalMilliSecondsAndTotalSeconds("-P01Y", "2012-01-01T00:00:00.000", -31536000000, -31536000);

	assertTotalMilliSecondsAndTotalSeconds("P01Y", "-2013-01-01T00:00:00.000", 31536000000, 31536000);
	assertTotalMilliSecondsAndTotalSeconds("P01Y", "-2012-01-01T00:00:00.000", 31622400000, 31622400);
	assertTotalMilliSecondsAndTotalSeconds("-P01Y", "-2012-01-01T00:00:00.000", -31536000000, -31536000);
	assertTotalMilliSecondsAndTotalSeconds("-P01Y", "-2011-01-01T00:00:00.000", -31622400000, -31622400);

	assertTotalMilliSecondsAndTotalSeconds("P01Y", "0-01-01T00:00:00.000", 31622400000, 31622400);
	assertTotalMilliSecondsAndTotalSeconds("P10Y", "0-01-01T00:00:00.000", 315619200000, 315619200);
	assertTotalMilliSecondsAndTotalSeconds("P100Y", "0-01-01T00:00:00.000", 3155760000000, 3155760000);
	assertTotalMilliSecondsAndTotalSeconds("P1000Y", "0-01-01T00:00:00.000", 31556995200000, 31556995200);
	assertTotalMilliSecondsAndTotalSeconds("P10000Y", "0-01-01T00:00:00.000", 315569520000000, 315569520000);
	assertTotalMilliSecondsAndTotalSeconds("P100000Y", "0-01-01T00:00:00.000", 3155695200000000, 3155695200000);
	assertTotalMilliSecondsAndTotalSeconds("P1000000Y", "0-01-01T00:00:00.000", 31556952000000000, 31556952000000);
	assertTotalMilliSecondsAndTotalSeconds("P10000000Y", "0-01-01T00:00:00.000", 315569520000000000, 315569520000000);
	assertTotalMilliSecondsAndTotalSeconds("P100000000Y", "0-01-01T00:00:00.000", 3155695200000000000, 3155695200000000);
//	assertTotalMilliSecondsAndTotalSeconds("P1000000000Y", "0-01-01T00:00:00.000", 31556952000000000000, 31556952000000000); //FAILS int64_t to small
	assertTotalMilliSecondsAndTotalSeconds("-P100000000Y", "0-01-01T00:00:00.000", -3155695200000000000, -3155695200000000);
	assertTotalMilliSecondsAndTotalSeconds("-P01Y", "0-01-01T00:00:00.000", -31536000000, -31536000);

	assertTotalMilliSecondsAndTotalSeconds("P42Y", "2014-10-23T14:53:50.589", 1325462400000, 1325462400);
	assertTotalMilliSecondsAndTotalSeconds("P42Y3M", "2014-10-23T14:53:50.589", 1333411200000, 1333411200);
	assertTotalMilliSecondsAndTotalSeconds("P42Y3M12D", "2014-10-23T14:53:50.589", 1334448000000, 1334448000);
	assertTotalMilliSecondsAndTotalSeconds("P42Y3M12DT02H", "2014-10-23T14:53:50.589", 1334455200000, 1334455200);
	assertTotalMilliSecondsAndTotalSeconds("P42Y3M12DT02H45M", "2014-10-23T14:53:50.589", 1334457900000, 1334457900);
	assertTotalMilliSecondsAndTotalSeconds("P42Y3M12DT02H45M13.000S", "2014-10-23T14:53:50.589", 1334457913000, 1334457913);
	assertTotalMilliSecondsAndTotalSeconds("P42Y3M12DT02H45M13.790S", "2014-10-23T14:53:50.589", 1334457913790, 1334457913);
}
END_TEST

START_TEST(test_addTimeDeltaToDate_ProlepticGregorian)
{
	assertAddTimeDeltaToDate("2014-11-27", "PT0.000S", "2014-11-27");
	assertAddTimeDeltaToDate("2014-11-27", "P1Y", "2015-11-27");
	assertAddTimeDeltaToDate("2014-11-27", "-P1Y", "2013-11-27");
	assertAddTimeDeltaToDate("2014-11-27", "P1M", "2014-12-27");
	assertAddTimeDeltaToDate("2014-11-27", "-P1M", "2014-10-27");
	assertAddTimeDeltaToDate("2014-11-27", "P1D", "2014-11-28");
	assertAddTimeDeltaToDate("2014-11-27", "-P1D", "2014-11-26");

	assertAddTimeDeltaToDate("2014-11-27", "P30D", "2014-12-27");
	assertAddTimeDeltaToDate("2014-11-27", "P31D", "2014-12-28");
	assertAddTimeDeltaToDate("2014-11-27", "P32D", "2014-12-29");
	assertAddTimeDeltaToDate("2014-01-31", "P1M", "2014-03-03");
	assertAddTimeDeltaToDate("2012-01-31", "P1M", "2012-03-02");
	assertAddTimeDeltaToDate("2014-03-31", "-P1M", "2014-03-03");
	assertAddTimeDeltaToDate("2012-03-31", "-P1M", "2012-03-02");
	assertAddTimeDeltaToDate("2014-11-27", "P127D", "2015-04-03");
	assertAddTimeDeltaToDate("2012-11-27", "P365D", "2013-11-27");
	assertAddTimeDeltaToDate("2011-11-27", "P365D", "2012-11-26");

	assertAddTimeDeltaToDate("0-01-01", "P2014Y10M26D", "2014-11-27");
	assertAddTimeDeltaToDate("2014-11-27", "-P2014Y10M26D", "0-01-01");
	assertAddTimeDeltaToDate("2014-11-27", "-P2014Y10M27D", "-1-12-31");

	assertAddTimeDeltaToDate("-2014-11-27", "PT0.000S", "-2014-11-27");
	assertAddTimeDeltaToDate("-2014-11-27", "P1Y", "-2013-11-27");
	assertAddTimeDeltaToDate("-2014-11-27", "-P1Y", "-2015-11-27");
	assertAddTimeDeltaToDate("-2014-11-27", "P1M", "-2014-12-27");
	assertAddTimeDeltaToDate("-2014-11-27", "-P1M", "-2014-10-27");
	assertAddTimeDeltaToDate("-2014-11-27", "P1D", "-2014-11-28");
	assertAddTimeDeltaToDate("-2014-11-27", "-P1D", "-2014-11-26");

	assertAddTimeDeltaToDate("-2014-11-27", "P30D", "-2014-12-27");
	assertAddTimeDeltaToDate("-2014-11-27", "P31D", "-2014-12-28");
	assertAddTimeDeltaToDate("-2014-11-27", "P32D", "-2014-12-29");
	assertAddTimeDeltaToDate("-2014-01-31", "P1M", "-2014-03-03");
	assertAddTimeDeltaToDate("-2012-01-31", "P1M", "-2012-03-02");
	assertAddTimeDeltaToDate("-2014-03-31", "-P1M", "-2014-03-03");
	assertAddTimeDeltaToDate("-2012-03-31", "-P1M", "-2012-03-02");
	assertAddTimeDeltaToDate("-2014-11-27", "P127D", "-2013-04-03");
	assertAddTimeDeltaToDate("-2013-11-27", "P365D", "-2012-11-26");
	assertAddTimeDeltaToDate("-2012-11-27", "P365D", "-2011-11-27");

	assertAddTimeDeltaToDate("-01-12-31", "-P2013Y1M4D", "-2014-11-27");
	assertAddTimeDeltaToDate("-2014-11-27", "P2013Y1M4D", "-1-12-31");
	assertAddTimeDeltaToDate("-2014-11-27", "P2013Y1M5D", "0-01-01");

	assertAddTimeDeltaToDate("-2147483648-01-01", "PT00.000S", "-2147483648-01-01");
	assertAddTimeDeltaToDate("-2147483647-01-01", "-P1Y", "-2147483648-01-01");
	assertAddTimeDeltaToDate("-2147483648-01-01", "P1Y", "-2147483647-01-01");
	assertAddTimeDeltaToDate("2147483647-01-01", "PT00.000S", "2147483647-01-01");
	assertAddTimeDeltaToDate("2147483646-01-01", "P1Y", "2147483647-01-01");
	assertAddTimeDeltaToDate("2147483647-01-01", "-P1Y", "2147483646-01-01");

	assertAddTimeDeltaToDate("1-01-01", "P2147483646Y", "2147483647-01-01");
	assertAddTimeDeltaToDate("0-01-01", "P2147483647Y", "2147483647-01-01");
	assertAddTimeDeltaToDate("2147483647-01-01", "-P2147483646Y", "1-01-01");
	assertAddTimeDeltaToDate("2147483647-01-01", "-P2147483647Y", "0-01-01");
}
END_TEST

START_TEST(test_addTimeDeltaToDate_YearOf365Days)
{
	assertAddTimeDeltaToDate("2014-11-27", "PT0.000S", "2014-11-27");
	assertAddTimeDeltaToDate("2014-11-27", "P1Y", "2015-11-27");
	assertAddTimeDeltaToDate("2014-11-27", "-P1Y", "2013-11-27");
	assertAddTimeDeltaToDate("2014-11-27", "P1M", "2014-12-27");
	assertAddTimeDeltaToDate("2014-11-27", "-P1M", "2014-10-27");
	assertAddTimeDeltaToDate("2014-11-27", "P1D", "2014-11-28");
	assertAddTimeDeltaToDate("2014-11-27", "-P1D", "2014-11-26");

	assertAddTimeDeltaToDate("2014-11-27", "P30D", "2014-12-27");
	assertAddTimeDeltaToDate("2014-11-27", "P31D", "2014-12-28");
	assertAddTimeDeltaToDate("2014-11-27", "P32D", "2014-12-29");
	assertAddTimeDeltaToDate("2014-01-31", "P1M", "2014-03-03");
	assertAddTimeDeltaToDate("2012-01-31", "P1M", "2012-03-03");
	assertAddTimeDeltaToDate("2014-03-31", "-P1M", "2014-03-03");
	assertAddTimeDeltaToDate("2012-03-31", "-P1M", "2012-03-03");
	assertAddTimeDeltaToDate("2014-11-27", "P127D", "2015-04-03");
	assertAddTimeDeltaToDate("2012-11-27", "P365D", "2013-11-27");
	assertAddTimeDeltaToDate("2011-11-27", "P365D", "2012-11-27");

	assertAddTimeDeltaToDate("0-01-01", "P2014Y10M26D", "2014-11-27");
	assertAddTimeDeltaToDate("2014-11-27", "-P2014Y10M26D", "0-01-01");
	assertAddTimeDeltaToDate("2014-11-27", "-P2014Y10M27D", "-1-12-31");

	assertAddTimeDeltaToDate("-2014-11-27", "PT0.000S", "-2014-11-27");
	assertAddTimeDeltaToDate("-2014-11-27", "P1Y", "-2013-11-27");
	assertAddTimeDeltaToDate("-2014-11-27", "-P1Y", "-2015-11-27");
	assertAddTimeDeltaToDate("-2014-11-27", "P1M", "-2014-12-27");
	assertAddTimeDeltaToDate("-2014-11-27", "-P1M", "-2014-10-27");
	assertAddTimeDeltaToDate("-2014-11-27", "P1D", "-2014-11-28");
	assertAddTimeDeltaToDate("-2014-11-27", "-P1D", "-2014-11-26");

	assertAddTimeDeltaToDate("-2014-11-27", "P30D", "-2014-12-27");
	assertAddTimeDeltaToDate("-2014-11-27", "P31D", "-2014-12-28");
	assertAddTimeDeltaToDate("-2014-11-27", "P32D", "-2014-12-29");
	assertAddTimeDeltaToDate("-2014-01-31", "P1M", "-2014-03-03");
	assertAddTimeDeltaToDate("-2012-01-31", "P1M", "-2012-03-03");
	assertAddTimeDeltaToDate("-2014-03-31", "-P1M", "-2014-03-03");
	assertAddTimeDeltaToDate("-2012-03-31", "-P1M", "-2012-03-03");
	assertAddTimeDeltaToDate("-2014-11-27", "P127D", "-2013-04-03");
	assertAddTimeDeltaToDate("-2013-11-27", "P365D", "-2012-11-27");
	assertAddTimeDeltaToDate("-2012-11-27", "P365D", "-2011-11-27");

	assertAddTimeDeltaToDate("-01-12-31", "-P2013Y1M4D", "-2014-11-27");
	assertAddTimeDeltaToDate("-2014-11-27", "P2013Y1M4D", "-1-12-31");
	assertAddTimeDeltaToDate("-2014-11-27", "P2013Y1M5D", "0-01-01");

	assertAddTimeDeltaToDate("-2147483648-01-01", "PT00.000S", "-2147483648-01-01");
	assertAddTimeDeltaToDate("-2147483647-01-01", "-P1Y", "-2147483648-01-01");
	assertAddTimeDeltaToDate("-2147483648-01-01", "P1Y", "-2147483647-01-01");
	assertAddTimeDeltaToDate("2147483647-01-01", "PT00.000S", "2147483647-01-01");
	assertAddTimeDeltaToDate("2147483646-01-01", "P1Y", "2147483647-01-01");
	assertAddTimeDeltaToDate("2147483647-01-01", "-P1Y", "2147483646-01-01");

	assertAddTimeDeltaToDate("1-01-01", "P2147483646Y", "2147483647-01-01");
	assertAddTimeDeltaToDate("0-01-01", "P2147483647Y", "2147483647-01-01");
	assertAddTimeDeltaToDate("2147483647-01-01", "-P2147483646Y", "1-01-01");
	assertAddTimeDeltaToDate("2147483647-01-01", "-P2147483647Y", "0-01-01");
}
END_TEST

START_TEST(test_addTimeDeltaToDate_YearOf360Days)
{
	assertAddTimeDeltaToDate("2014-11-27", "PT0.000S", "2014-11-27");
	assertAddTimeDeltaToDate("2014-11-27", "P1Y", "2015-11-27");
	assertAddTimeDeltaToDate("2014-11-27", "-P1Y", "2013-11-27");
	assertAddTimeDeltaToDate("2014-11-27", "P1M", "2014-12-27");
	assertAddTimeDeltaToDate("2014-11-27", "-P1M", "2014-10-27");
	assertAddTimeDeltaToDate("2014-11-27", "P1D", "2014-11-28");
	assertAddTimeDeltaToDate("2014-11-27", "-P1D", "2014-11-26");

	assertAddTimeDeltaToDate("2014-11-27", "P30D", "2014-12-27");
	assertAddTimeDeltaToDate("2014-11-27", "P31D", "2014-12-28");
	assertAddTimeDeltaToDate("2014-11-27", "P32D", "2014-12-29");
	assertAddTimeDeltaToDate("2014-01-30", "P1M", "2014-02-30");
	assertAddTimeDeltaToDate("2012-01-30", "P1M", "2012-02-30");
	assertAddTimeDeltaToDate("2014-03-30", "-P1M", "2014-02-30");
	assertAddTimeDeltaToDate("2012-03-30", "-P1M", "2012-02-30");
	assertAddTimeDeltaToDate("2014-11-27", "P127D", "2015-04-04");
	assertAddTimeDeltaToDate("2012-11-27", "P360D", "2013-11-27");
	assertAddTimeDeltaToDate("2011-11-27", "P360D", "2012-11-27");

	assertAddTimeDeltaToDate("0-01-01", "P2014Y10M26D", "2014-11-27");
	assertAddTimeDeltaToDate("2014-11-27", "-P2014Y10M26D", "0-01-01");
	assertAddTimeDeltaToDate("2014-11-27", "-P2014Y10M27D", "-1-12-30");

	assertAddTimeDeltaToDate("-2014-11-27", "PT0.000S", "-2014-11-27");
	assertAddTimeDeltaToDate("-2014-11-27", "P1Y", "-2013-11-27");
	assertAddTimeDeltaToDate("-2014-11-27", "-P1Y", "-2015-11-27");
	assertAddTimeDeltaToDate("-2014-11-27", "P1M", "-2014-12-27");
	assertAddTimeDeltaToDate("-2014-11-27", "-P1M", "-2014-10-27");
	assertAddTimeDeltaToDate("-2014-11-27", "P1D", "-2014-11-28");
	assertAddTimeDeltaToDate("-2014-11-27", "-P1D", "-2014-11-26");

	assertAddTimeDeltaToDate("-2014-11-27", "P30D", "-2014-12-27");
	assertAddTimeDeltaToDate("-2014-11-27", "P31D", "-2014-12-28");
	assertAddTimeDeltaToDate("-2014-11-27", "P32D", "-2014-12-29");
	assertAddTimeDeltaToDate("-2014-01-30", "P1M", "-2014-02-30");
	assertAddTimeDeltaToDate("-2012-01-30", "P1M", "-2012-02-30");
	assertAddTimeDeltaToDate("-2014-03-30", "-P1M", "-2014-02-30");
	assertAddTimeDeltaToDate("-2012-03-30", "-P1M", "-2012-02-30");
	assertAddTimeDeltaToDate("-2014-11-27", "P127D", "-2013-04-04");
	assertAddTimeDeltaToDate("-2013-11-27", "P360D", "-2012-11-27");
	assertAddTimeDeltaToDate("-2012-11-27", "P360D", "-2011-11-27");

	assertAddTimeDeltaToDate("-01-12-30", "-P2013Y1M4D", "-2014-11-26");
	assertAddTimeDeltaToDate("-2014-11-27", "P2013Y1M3D", "-1-12-30");
	assertAddTimeDeltaToDate("-2014-11-27", "P2013Y1M4D", "0-01-01");

	assertAddTimeDeltaToDate("-2147483648-01-01", "PT00.000S", "-2147483648-01-01");
	assertAddTimeDeltaToDate("-2147483647-01-01", "-P1Y", "-2147483648-01-01");
	assertAddTimeDeltaToDate("-2147483648-01-01", "P1Y", "-2147483647-01-01");
	assertAddTimeDeltaToDate("2147483647-01-01", "PT00.000S", "2147483647-01-01");
	assertAddTimeDeltaToDate("2147483646-01-01", "P1Y", "2147483647-01-01");
	assertAddTimeDeltaToDate("2147483647-01-01", "-P1Y", "2147483646-01-01");

	assertAddTimeDeltaToDate("1-01-01", "P2147483646Y", "2147483647-01-01");
	assertAddTimeDeltaToDate("0-01-01", "P2147483647Y", "2147483647-01-01");
	assertAddTimeDeltaToDate("2147483647-01-01", "-P2147483646Y", "1-01-01");
	assertAddTimeDeltaToDate("2147483647-01-01", "-P2147483647Y", "0-01-01");
}
END_TEST

START_TEST(test_addTimeDeltaToDateTime)
{
	fail("Test not yet implemented");
}
END_TEST

START_TEST(test_moduloTimeDeltaFromDateTime)
{
	fail("Test not yet implemented");
}
END_TEST

START_TEST(test_elementwiseScalarMultiplyTimeDelta)
{
	fail("Test not yet implemented");
}
END_TEST

START_TEST(test_elementwiseAddTimeDeltatoTimeDelta)
{
	fail("Test not yet implemented");
}
END_TEST

START_TEST(test_PTStringFunctions)
{
	fail("Test not yet implemented");
}
END_TEST

static void setup_ProlepticGregorian(void)
{
	initCalendar(PROLEPTIC_GREGORIAN);
}

static void setup_YearOf365Days(void)
{
	initCalendar(YEAR_OF_365_DAYS);
}

static void setup_YearOf360Days(void)
{
	initCalendar(YEAR_OF_360_DAYS);
}

static void teardown(void)
{
	freeCalendar();
}

void add_mtime_timedelta_test_to_suite(Suite* suite)
{
	TCase *tcase_ProlepticGregorian = tcase_create("mtime_time_test_ProlepticGregorian");
	suite_add_tcase(suite, tcase_ProlepticGregorian);
	tcase_add_checked_fixture(tcase_ProlepticGregorian, setup_ProlepticGregorian, teardown);
	tcase_add_test(tcase_ProlepticGregorian, test_create_timedelta_from_valid_strings);
	tcase_add_test(tcase_ProlepticGregorian, test_invalid_strings);
	tcase_add_test(tcase_ProlepticGregorian, test_constructAndCopyTimeDelta);
	tcase_add_test(tcase_ProlepticGregorian, test_replaceTimeDelta);
	tcase_add_test(tcase_ProlepticGregorian, test_timeDeltaToJulianDelta_ProlepticGregorian_1);
	tcase_add_test(tcase_ProlepticGregorian, test_timeDeltaToJulianDelta_ProlepticGregorian_2);
	tcase_add_test(tcase_ProlepticGregorian, test_timeDeltaToJulianDelta_ProlepticGregorian_3);
	tcase_add_test(tcase_ProlepticGregorian, test_timeDeltaToJulianDelta_highDeltas);
//	tcase_add_test(tcase_ProlepticGregorian, test_julianDeltaToTimeDelta);
	tcase_add_test(tcase_ProlepticGregorian, test_divideTimeDeltaInSeconds);
	tcase_add_test(tcase_ProlepticGregorian, test_getTimeDeltaFromDate_ProlepticGregorian);
	tcase_add_test(tcase_ProlepticGregorian, test_getTimeDeltaFromDate_ProlepticGregorian_LeapYears);
	tcase_add_test(tcase_ProlepticGregorian, test_getTimeDeltaFromDate_ProlepticGregorian_HighDeltas);
	tcase_add_test(tcase_ProlepticGregorian, test_getTimeDeltaFromDateTime_ProlepticGregorian);
	tcase_add_test(tcase_ProlepticGregorian, test_getTimeDeltaFromDateTime_ProlepticGregorian_HighDeltas);
	tcase_add_test(tcase_ProlepticGregorian, test_getTotalMilliSecondsTimeDelta_and_getTotalSecondsTimeDelta);
	tcase_add_test(tcase_ProlepticGregorian, test_addTimeDeltaToDate_ProlepticGregorian);
//	tcase_add_test(tcase_ProlepticGregorian, test_addTimeDeltaToDateTime);
//	tcase_add_test(tcase_ProlepticGregorian, test_moduloTimeDeltaFromDateTime);
//	tcase_add_test(tcase_ProlepticGregorian, test_elementwiseScalarMultiplyTimeDelta);
//	tcase_add_test(tcase_ProlepticGregorian, test_elementwiseAddTimeDeltatoTimeDelta);
//	tcase_add_test(tcase_ProlepticGregorian, test_PTStringFunctions);

	TCase *tcase_YearOf365Days = tcase_create("mtime_time_test_YearOf365Days");
	suite_add_tcase(suite, tcase_YearOf365Days);
	tcase_add_checked_fixture(tcase_YearOf365Days, setup_YearOf365Days, teardown);
	tcase_add_test(tcase_YearOf365Days, test_timeDeltaToJulianDelta_YearOf365Days);
	tcase_add_test(tcase_YearOf365Days, test_getTimeDeltaFromDate_YearOf365Days);
	tcase_add_test(tcase_YearOf365Days, test_getTimeDeltaFromDate_YearOf365Days_HighDeltas);
	tcase_add_test(tcase_YearOf365Days, test_getTimeDeltaFromDateTime_YearOf365Days);
	tcase_add_test(tcase_YearOf365Days, test_addTimeDeltaToDate_YearOf365Days);

	TCase *tcase_YearOf360Days = tcase_create("mtime_time_test_YearOf360Days");
	suite_add_tcase(suite, tcase_YearOf360Days);
	tcase_add_checked_fixture(tcase_YearOf360Days, setup_YearOf360Days, teardown);
	tcase_add_test(tcase_YearOf360Days, test_timeDeltaToJulianDelta_YearOf360Days);
	tcase_add_test(tcase_YearOf360Days, test_getTimeDeltaFromDate_YearOf360Days);
	tcase_add_test(tcase_YearOf360Days, test_getTimeDeltaFromDate_YearOf360Days_HighDeltas);
	tcase_add_test(tcase_YearOf360Days, test_getTimeDeltaFromDateTime_YearOf360Days);
	tcase_add_test(tcase_YearOf360Days, test_addTimeDeltaToDate_YearOf360Days);
}

/*** SPECIAL ASSERT FUNCTIONS ***/

void assertTimeDelta(const char* input_string, bool std_form, char sign, int64_t year, int month, int day, int hour, int minute, int second, int ms, const char* expected_output_string)
{
	const char* format = "Parsing of timedelta string \"%s\" failed.";
	size_t length = snprintf(NULL, 0, format, input_string) + 1;
	char* msg = malloc(length);
	snprintf(msg, length, format, input_string);

	char tmp[MAX_DATE_STR_LEN];

	TimeDelta timedelta = newTimeDelta(input_string);
	ck_assert_msg(timedelta != NULL, msg);
	ck_assert_str_eq(expected_output_string, timedeltaToString(timedelta, tmp));
	ck_assert_str_eq(expected_output_string, tmp);
	ck_assert_int_eq(std_form, timedelta->flag_std_form);
	ck_assert_int_eq(sign, timedelta->sign);
	ck_assert_int_eq(year, timedelta->year);
	ck_assert_int_eq(month, timedelta->month);
	ck_assert_int_eq(day, timedelta->day);
	ck_assert_int_eq(hour, timedelta->hour);
	ck_assert_int_eq(minute, timedelta->minute);
	ck_assert_int_eq(second, timedelta->second);
	ck_assert_int_eq(ms, timedelta->ms);

	TimeDelta raw_timedelta = newRawTimeDelta(sign, year, month, day, hour, minute, second, ms);
	ck_assert(raw_timedelta != NULL);
	ck_assert_str_eq(expected_output_string, timedeltaToString(raw_timedelta, tmp));
	ck_assert_int_eq(std_form, timedelta->flag_std_form);
	ck_assert_int_eq(sign, raw_timedelta->sign);
	ck_assert_int_eq(year, raw_timedelta->year);
	ck_assert_int_eq(month, raw_timedelta->month);
	ck_assert_int_eq(day, raw_timedelta->day);
	ck_assert_int_eq(hour, raw_timedelta->hour);
	ck_assert_int_eq(minute, raw_timedelta->minute);
	ck_assert_int_eq(second, raw_timedelta->second);
	ck_assert_int_eq(ms, raw_timedelta->ms);

	TimeDelta timedelta_from_string = newTimeDelta(timedeltaToString(raw_timedelta, tmp));
	ck_assert(timedelta_from_string != NULL);
	ck_assert_str_eq(expected_output_string, timedeltaToString(timedelta_from_string, tmp));

	deallocateTimeDelta(timedelta);
	free(msg);
}

void assertInvalidTimeDelta(const char* input_string)
{
	TimeDelta timedelta = newTimeDelta(input_string);

	if (timedelta != NULL)
	{
		const char* format = "Invalid timedelta string \"%s\" didn't result in null pointer but in timedelta \"%s\".";
		char output_string[MAX_TIMEDELTA_STR_LEN];
		timedeltaToString(timedelta, output_string);

		size_t length = snprintf(NULL, 0, format, input_string, output_string) + 1;
		char* msg = malloc(length);
		snprintf(msg, length, format, input_string, output_string);
		deallocateTimeDelta(timedelta);

		ck_abort_msg(msg);
		free(msg); //Never executed?
	}
}

void assertTimeDeltaToJulianDelta(const char* timedelta_string, const char* base_datetime_string, char expected_jd_sign, int64_t expected_jd_day, int64_t expected_jd_ms)
{
	TimeDelta timedelta = newTimeDelta(timedelta_string);
	ck_assert(timedelta != NULL);

	DateTime datetime = newDateTime(base_datetime_string);
	ck_assert(datetime != NULL);

	JulianDelta juliandeltaTemp = newJulianDelta('+', 0, 0);
	JulianDelta juliandelta = timeDeltaToJulianDelta(timedelta, datetime, juliandeltaTemp);
	ck_assert(datetime != NULL);
	ck_assert(juliandeltaTemp == juliandelta);

	const char* format = "timeDeltaToJulianDelta failed for \"%s\" + \"%s\": expected:(%c, %lld, %lld) actual:(%c, %lld, %lld).";
	size_t length = snprintf(NULL, 0, format, base_datetime_string, timedelta_string, expected_jd_sign, expected_jd_day, expected_jd_ms, juliandelta->sign, juliandelta->day, juliandelta->ms) + 1;
	char* msg = malloc(length);
	snprintf(msg, length, format, base_datetime_string, timedelta_string, expected_jd_sign, expected_jd_day, expected_jd_ms, juliandelta->sign, juliandelta->day, juliandelta->ms);

	ck_assert_msg(expected_jd_sign == juliandelta->sign, msg);
	ck_assert_msg(expected_jd_day == juliandelta->day, msg);
	ck_assert_msg(expected_jd_ms == juliandelta->ms, msg);

	deallocateTimeDelta(timedelta);
	deallocateDateTime(datetime);
	deallocateJulianDelta(juliandelta);
	free(msg);
}

void assertTimeDeltaToJulianDeltaWithoutExpactations(const char* timedelta_string, const char* base_datetime_string, char expected_jd_sign)
{
	TimeDelta timedelta = newTimeDelta(timedelta_string);
	ck_assert(timedelta != NULL);

	DateTime datetime = newDateTime(base_datetime_string);
	ck_assert(datetime != NULL);

	JulianDelta juliandeltaTemp = newJulianDelta('+', 0, 0);
	JulianDelta juliandelta = timeDeltaToJulianDelta(timedelta, datetime, juliandeltaTemp);
	ck_assert(juliandelta != NULL);
	ck_assert(juliandeltaTemp == juliandelta);

	ck_assert(expected_jd_sign == juliandelta->sign);

	deallocateTimeDelta(timedelta);
	deallocateDateTime(datetime);
	deallocateJulianDelta(juliandelta);
}

void assertJulianDeltaToTimeDelta(int64_t jd_day, int64_t jd_ms, const char* base_datetime_string, const char* expected_timedelta_string)
{
	char jd_sign = jd_day < 0 ? '-' : '+';

	JulianDelta juliandelta = newJulianDelta(jd_sign, jd_day, jd_ms);
	ck_assert(juliandelta != NULL);

	DateTime datetime = newDateTime(base_datetime_string);
	ck_assert(datetime != NULL);

	TimeDelta timedeltaTemp = newTimeDelta("P0Y");
	TimeDelta timedelta = julianDeltaToTimeDelta(juliandelta, datetime, timedeltaTemp);
	ck_assert(timedelta != NULL);
	ck_assert(timedeltaTemp == timedelta);
	char timedelta_string[MAX_TIMEDELTA_STR_LEN];
	timedeltaToString(timedelta, timedelta_string);

	const char* format = "julianDeltaToTimeDelta failed for \"%s\" + (%c, %dl, %dl): expected:\"%s\" actual:\"%s\".";
	size_t length = snprintf(NULL, 0, format, base_datetime_string, jd_sign, jd_day, jd_ms, expected_timedelta_string, timedelta_string) + 1;
	char* msg = malloc(length);
	snprintf(msg, length, format, base_datetime_string, jd_sign, jd_day, jd_ms, expected_timedelta_string, timedelta_string);

	ck_assert_msg(strcmp(expected_timedelta_string, timedelta_string) == 0, msg);

	deallocateJulianDelta(juliandelta);
	deallocateDateTime(datetime);
	deallocateTimeDelta(timedelta);
	free(msg);
}

void assertGetTimeDeltaFromDate(const char* date1_string, const char* date2_string, const char* expected_timedelta_string)
{
	Date date1 = newDate(date1_string);
	ck_assert(date1 != NULL);
	char date1_string_temp[MAX_DATE_STR_LEN];
	ck_assert_str_eq(date1_string, dateToString(date1, date1_string_temp));

	Date date2 = newDate(date2_string);
	ck_assert(date2 != NULL);
	char date2_string_temp[MAX_DATE_STR_LEN];
	ck_assert_str_eq(date2_string, dateToString(date2, date2_string_temp));

	TimeDelta timedeltaTemp = newTimeDelta("PT00.000S");
	TimeDelta timedelta = getTimeDeltaFromDate(date1, date2, timedeltaTemp);
	ck_assert(timedelta != NULL);
	ck_assert(timedeltaTemp == timedelta);
	char timedelta_string[MAX_TIMEDELTA_STR_LEN];
	timedeltaToString(timedelta, timedelta_string);

	const char* format = "getTimeDeltaFromDate failed for \"%s\" - \"%s\": expected:\"%s\" actual:\"%s\".";
	size_t length = snprintf(NULL, 0, format, date1_string, date2_string, expected_timedelta_string, timedelta_string) + 1;
	char* msg = malloc(length);
	snprintf(msg, length, format, date1_string, date2_string, expected_timedelta_string, timedelta_string);

	ck_assert_msg(strcmp(expected_timedelta_string, timedelta_string) == 0, msg);

	deallocateDate(date1);
	deallocateDate(date2);
	deallocateTimeDelta(timedelta);
	free(msg);
}

void assertGetTimeDeltaFromDateTime(const char* datetime1_string, const char* datetime2_string, const char* expected_timedelta_string)
{
	DateTime datetime1 = newDateTime(datetime1_string);
	ck_assert(datetime1 != NULL);
	char datetime1_string_temp[MAX_DATETIME_STR_LEN];
	ck_assert_str_eq(datetime1_string, datetimeToString(datetime1, datetime1_string_temp));

	DateTime datetime2 = newDateTime(datetime2_string);
	ck_assert(datetime2 != NULL);
	char datetime2_string_temp[MAX_DATETIME_STR_LEN];
	ck_assert_str_eq(datetime2_string, datetimeToString(datetime2, datetime2_string_temp));

	TimeDelta timedeltaTemp = newTimeDelta("PT00.000S");
	TimeDelta timedelta = getTimeDeltaFromDateTime(datetime1, datetime2, timedeltaTemp);
	ck_assert(timedelta != NULL);
	ck_assert(timedeltaTemp == timedelta);
	char timedelta_string[MAX_TIMEDELTA_STR_LEN];
	timedeltaToString(timedelta, timedelta_string);

	const char* format = "getTimeDeltaFromDateTime failed for \"%s\" - \"%s\": expected:\"%s\" actual:\"%s\".";
	size_t length = snprintf(NULL, 0, format, datetime1_string, datetime2_string, expected_timedelta_string, timedelta_string) + 1;
	char* msg = malloc(length);
	snprintf(msg, length, format, datetime1_string, datetime2_string, expected_timedelta_string, timedelta_string);

	ck_assert_msg(strcmp(expected_timedelta_string, timedelta_string) == 0, msg);

	deallocateDateTime(datetime1);
	deallocateDateTime(datetime2);
	deallocateTimeDelta(timedelta);
	free(msg);
}

void assertDivideTimeDeltaInSeconds(const char* timedelta_dividend_string, const char* timedelta_divisor_string, int64_t expected_quotient, int64_t expected_remainder)
{
	TimeDelta dividend = newTimeDelta(timedelta_dividend_string);
	ck_assert(dividend != NULL);

	TimeDelta divisor = newTimeDelta(timedelta_divisor_string);
	ck_assert(divisor != NULL);

	DivisionQuotientTimeDelta quotientTemp = (DivisionQuotientTimeDelta) calloc(1, sizeof(DivisionQuotientTimeDelta));
	DivisionQuotientTimeDelta quotient = divideTimeDeltaInSeconds(dividend, divisor, quotientTemp);
	ck_assert(quotient != NULL);
	ck_assert(quotientTemp == quotient);

	const char* format = "divideTimeDeltaInSeconds failed for %s/%s: expected:(%lld, %lld) actual:(%lld, %lld).";
	size_t length = snprintf(NULL, 0, format, timedelta_dividend_string, timedelta_divisor_string, expected_quotient, expected_remainder, quotient->quotient, quotient->remainder_in_ms) + 1;
	char* msg = malloc(length);
	snprintf(msg, length, format, timedelta_dividend_string, timedelta_divisor_string, expected_quotient, expected_remainder, quotient->quotient, quotient->remainder_in_ms);

	ck_assert_msg(expected_quotient == quotient->quotient, msg);
	ck_assert_msg(expected_remainder == quotient->remainder_in_ms, msg);

	deallocateTimeDelta(dividend);
	deallocateTimeDelta(divisor);
	free(quotientTemp);
	quotientTemp = NULL;
}


void assertTotalMilliSecondsAndTotalSeconds(const char* timedelta_string, const char* base_datetime_string, int64_t expected_milliseconds, int64_t expected_seconds)
{
	TimeDelta timedelta = newTimeDelta(timedelta_string);
	ck_assert(timedelta != NULL);

	DateTime datetime = newDateTime(base_datetime_string);
	ck_assert(datetime != NULL);

	int64_t milliseconds = getTotalMilliSecondsTimeDelta(timedelta, datetime);
	int64_t seconds = getTotalSecondsTimeDelta(timedelta, datetime);

	const char* formatMS = "getTotalMilliSecondsTimeDelta failed for \"%s\" + \"%s\": expected: %lld ms; actual: %lld ms.";
	size_t lengthMS = snprintf(NULL, 0, formatMS, base_datetime_string, timedelta_string, expected_milliseconds, milliseconds) + 1;
	char* msgMS = malloc(lengthMS);
	snprintf(msgMS, lengthMS, formatMS, base_datetime_string, timedelta_string, expected_milliseconds, milliseconds);

	ck_assert_msg(expected_milliseconds == milliseconds, msgMS);

	const char* formatS = "getTotalSecondsTimeDelta failed for \"%s\" + \"%s\": expected: %lld s; actual: %lld s.";
	size_t lengthS = snprintf(NULL, 0, formatS, base_datetime_string, timedelta_string, expected_seconds, seconds) + 1;
	char* msgS = malloc(lengthS);
	snprintf(msgS, lengthS, formatS, base_datetime_string, timedelta_string, expected_seconds, seconds);

	ck_assert_msg(expected_seconds == seconds, msgS);

	deallocateTimeDelta(timedelta);
	deallocateDateTime(datetime);
	free(msgMS);
	free(msgS);
}


void assertAddTimeDeltaToDate(const char* date_string, const char* timedelta_string, const char* expected_date_string)
{
	Date date = newDate(date_string);
	ck_assert(date != NULL);

	TimeDelta timedelta = newTimeDelta(timedelta_string);
	ck_assert(timedelta != NULL);

	Date dateResultTemp = newTimeDelta("PT00.000S");
	Date dateResult = addTimeDeltaToDate(date, timedelta, dateResultTemp);
	ck_assert(dateResult != NULL);
	ck_assert(dateResultTemp == dateResult);
	char result_string[MAX_DATE_STR_LEN];
	dateToString(dateResult, result_string);

	const char* format = "addTimeDeltaToDate failed for \"%s\" + \"%s\": expected:\"%s\" actual:\"%s\".";
	size_t length = snprintf(NULL, 0, format, date_string, timedelta_string, expected_date_string, result_string) + 1;
	char* msg = malloc(length);
	snprintf(msg, length, format, date_string, timedelta_string, expected_date_string, result_string);

	ck_assert_msg(strcmp(expected_date_string, result_string) == 0, msg);

	deallocateDate(date);
	deallocateTimeDelta(timedelta);
	deallocateDate(dateResult);
	free(msg);
}
